"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import release_policy_pb2
import secret_store_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _DataRoomStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _DataRoomStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_DataRoomStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    Active: _DataRoomStatus.ValueType  # 0
    Stopped: _DataRoomStatus.ValueType  # 1

class DataRoomStatus(_DataRoomStatus, metaclass=_DataRoomStatusEnumTypeWrapper): ...

Active: DataRoomStatus.ValueType  # 0
Stopped: DataRoomStatus.ValueType  # 1
global___DataRoomStatus = DataRoomStatus

@typing_extensions.final
class ArchV2Request(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATEPOLICY_FIELD_NUMBER: builtins.int
    SECRETSTORE_FIELD_NUMBER: builtins.int
    UPDATESTATUS_FIELD_NUMBER: builtins.int
    GETVERIFICATIONKEY_FIELD_NUMBER: builtins.int
    CREATEDATAROOM_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOM_FIELD_NUMBER: builtins.int
    @property
    def createPolicy(self) -> global___CreatePolicyRequest: ...
    @property
    def secretStore(self) -> secret_store_pb2.SecretStoreRequest: ...
    @property
    def updateStatus(self) -> global___UpdateDataRoomStatusRequest: ...
    @property
    def getVerificationKey(self) -> global___GetVerificationKeyRequest: ...
    @property
    def createDataRoom(self) -> global___CreateDataRoomRequest:
        """TODO: Remove this once we can create DCRs with api-platform"""
    @property
    def retrieveDataRoom(self) -> global___RetrieveDataRoomRequest: ...
    def __init__(
        self,
        *,
        createPolicy: global___CreatePolicyRequest | None = ...,
        secretStore: secret_store_pb2.SecretStoreRequest | None = ...,
        updateStatus: global___UpdateDataRoomStatusRequest | None = ...,
        getVerificationKey: global___GetVerificationKeyRequest | None = ...,
        createDataRoom: global___CreateDataRoomRequest | None = ...,
        retrieveDataRoom: global___RetrieveDataRoomRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "getVerificationKey", b"getVerificationKey", "request", b"request", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "getVerificationKey", b"getVerificationKey", "request", b"request", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request", b"request"]) -> typing_extensions.Literal["createPolicy", "secretStore", "updateStatus", "getVerificationKey", "createDataRoom", "retrieveDataRoom"] | None: ...

global___ArchV2Request = ArchV2Request

@typing_extensions.final
class ArchV2Response(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FAILURE_FIELD_NUMBER: builtins.int
    CREATEPOLICY_FIELD_NUMBER: builtins.int
    SECRETSTORE_FIELD_NUMBER: builtins.int
    UPDATESTATUS_FIELD_NUMBER: builtins.int
    GETVERIFICATIONKEY_FIELD_NUMBER: builtins.int
    CREATEDATAROOM_FIELD_NUMBER: builtins.int
    RETRIEVEDATAROOM_FIELD_NUMBER: builtins.int
    failure: builtins.str
    @property
    def createPolicy(self) -> global___CreatePolicyResponse: ...
    @property
    def secretStore(self) -> secret_store_pb2.SecretStoreResponse: ...
    @property
    def updateStatus(self) -> global___UpdateDataRoomStatusResponse: ...
    @property
    def getVerificationKey(self) -> global___GetVerificationKeyResponse: ...
    @property
    def createDataRoom(self) -> global___CreateDataRoomResponse: ...
    @property
    def retrieveDataRoom(self) -> global___RetrieveDataRoomResponse: ...
    def __init__(
        self,
        *,
        failure: builtins.str = ...,
        createPolicy: global___CreatePolicyResponse | None = ...,
        secretStore: secret_store_pb2.SecretStoreResponse | None = ...,
        updateStatus: global___UpdateDataRoomStatusResponse | None = ...,
        getVerificationKey: global___GetVerificationKeyResponse | None = ...,
        createDataRoom: global___CreateDataRoomResponse | None = ...,
        retrieveDataRoom: global___RetrieveDataRoomResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "failure", b"failure", "getVerificationKey", b"getVerificationKey", "response", b"response", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["createDataRoom", b"createDataRoom", "createPolicy", b"createPolicy", "failure", b"failure", "getVerificationKey", b"getVerificationKey", "response", b"response", "retrieveDataRoom", b"retrieveDataRoom", "secretStore", b"secretStore", "updateStatus", b"updateStatus"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response", b"response"]) -> typing_extensions.Literal["failure", "createPolicy", "secretStore", "updateStatus", "getVerificationKey", "createDataRoom", "retrieveDataRoom"] | None: ...

global___ArchV2Response = ArchV2Response

@typing_extensions.final
class GetVerificationKeyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___GetVerificationKeyRequest = GetVerificationKeyRequest

@typing_extensions.final
class GetVerificationKeyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERIFICATIONKEY_FIELD_NUMBER: builtins.int
    verificationKey: builtins.bytes
    def __init__(
        self,
        *,
        verificationKey: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["verificationKey", b"verificationKey"]) -> None: ...

global___GetVerificationKeyResponse = GetVerificationKeyResponse

@typing_extensions.final
class CreateDataRoomRequest(google.protobuf.message.Message):
    """TODO: Remove this once we can create DCRs with api-platform"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    DATAROOMMETADATA_FIELD_NUMBER: builtins.int
    dataRoom: builtins.str
    dataRoomMetadata: builtins.bytes
    def __init__(
        self,
        *,
        dataRoom: builtins.str = ...,
        dataRoomMetadata: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "dataRoomMetadata", b"dataRoomMetadata"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata", "dataRoom", b"dataRoom", "dataRoomMetadata", b"dataRoomMetadata"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_dataRoomMetadata", b"_dataRoomMetadata"]) -> typing_extensions.Literal["dataRoomMetadata"] | None: ...

global___CreateDataRoomRequest = CreateDataRoomRequest

@typing_extensions.final
class CreateDataRoomResponse(google.protobuf.message.Message):
    """TODO: Remove this once we can create DCRs with api-platform"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    DATAROOMVALIDATIONERROR_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    @property
    def dataRoomValidationError(self) -> global___DataRoomValidationError: ...
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
        dataRoomValidationError: global___DataRoomValidationError | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError", "response", b"response"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "dataRoomValidationError", b"dataRoomValidationError", "response", b"response"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["response", b"response"]) -> typing_extensions.Literal["dataRoomId", "dataRoomValidationError"] | None: ...

global___CreateDataRoomResponse = CreateDataRoomResponse

@typing_extensions.final
class CreatePolicyRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POLICY_FIELD_NUMBER: builtins.int
    SCOPE_FIELD_NUMBER: builtins.int
    @property
    def policy(self) -> release_policy_pb2.ReleasePolicy: ...
    scope: builtins.bytes
    """/ Required if referencing a dataset as part of the policy"""
    def __init__(
        self,
        *,
        policy: release_policy_pb2.ReleasePolicy | None = ...,
        scope: builtins.bytes | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_scope", b"_scope", "policy", b"policy", "scope", b"scope"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_scope", b"_scope", "policy", b"policy", "scope", b"scope"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_scope", b"_scope"]) -> typing_extensions.Literal["scope"] | None: ...

global___CreatePolicyRequest = CreatePolicyRequest

@typing_extensions.final
class CreatePolicyResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POLICYID_FIELD_NUMBER: builtins.int
    policyId: builtins.str
    def __init__(
        self,
        *,
        policyId: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["policyId", b"policyId"]) -> None: ...

global___CreatePolicyResponse = CreatePolicyResponse

@typing_extensions.final
class UpdateDataRoomStatusRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.str
    status: global___DataRoomStatus.ValueType
    def __init__(
        self,
        *,
        dataRoomId: builtins.str = ...,
        status: global___DataRoomStatus.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId", "status", b"status"]) -> None: ...

global___UpdateDataRoomStatusRequest = UpdateDataRoomStatusRequest

@typing_extensions.final
class UpdateDataRoomStatusResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UpdateDataRoomStatusResponse = UpdateDataRoomStatusResponse

@typing_extensions.final
class RetrieveDataRoomRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOMID_FIELD_NUMBER: builtins.int
    dataRoomId: builtins.bytes
    def __init__(
        self,
        *,
        dataRoomId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoomId", b"dataRoomId"]) -> None: ...

global___RetrieveDataRoomRequest = RetrieveDataRoomRequest

@typing_extensions.final
class RetrieveDataRoomResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATAROOM_FIELD_NUMBER: builtins.int
    dataRoom: builtins.str
    """/ JSON string of the HL representation"""
    def __init__(
        self,
        *,
        dataRoom: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dataRoom", b"dataRoom"]) -> None: ...

global___RetrieveDataRoomResponse = RetrieveDataRoomResponse

@typing_extensions.final
class DataRoomValidationError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    PERMISSIONINDEX_FIELD_NUMBER: builtins.int
    COMPUTENODEID_FIELD_NUMBER: builtins.int
    USERPERMISSIONID_FIELD_NUMBER: builtins.int
    ATTESTATIONSPECIFICATIONID_FIELD_NUMBER: builtins.int
    AUTHENTICATIONMETHODID_FIELD_NUMBER: builtins.int
    message: builtins.str
    permissionIndex: builtins.int
    computeNodeId: builtins.str
    userPermissionId: builtins.str
    attestationSpecificationId: builtins.str
    authenticationMethodId: builtins.str
    def __init__(
        self,
        *,
        message: builtins.str = ...,
        permissionIndex: builtins.int | None = ...,
        computeNodeId: builtins.str | None = ...,
        userPermissionId: builtins.str | None = ...,
        attestationSpecificationId: builtins.str | None = ...,
        authenticationMethodId: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId", "_authenticationMethodId", b"_authenticationMethodId", "_computeNodeId", b"_computeNodeId", "_permissionIndex", b"_permissionIndex", "_userPermissionId", b"_userPermissionId", "attestationSpecificationId", b"attestationSpecificationId", "authenticationMethodId", b"authenticationMethodId", "computeNodeId", b"computeNodeId", "message", b"message", "permissionIndex", b"permissionIndex", "userPermissionId", b"userPermissionId"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_attestationSpecificationId", b"_attestationSpecificationId"]) -> typing_extensions.Literal["attestationSpecificationId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_authenticationMethodId", b"_authenticationMethodId"]) -> typing_extensions.Literal["authenticationMethodId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_computeNodeId", b"_computeNodeId"]) -> typing_extensions.Literal["computeNodeId"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_permissionIndex", b"_permissionIndex"]) -> typing_extensions.Literal["permissionIndex"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_userPermissionId", b"_userPermissionId"]) -> typing_extensions.Literal["userPermissionId"] | None: ...

global___DataRoomValidationError = DataRoomValidationError
