# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: meta_sink.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'meta_sink.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fmeta_sink.proto\x12\tmeta_sink\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"C\n\x07ZipFile\x12+\n\nsingleFile\x18\x01 \x01(\x0b\x32\x15.meta_sink.SingleFileH\x00\x42\x0b\n\tselection\"{\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12!\n\x03raw\x18\x03 \x01(\x0b\x32\x12.meta_sink.RawFileH\x00\x12!\n\x03zip\x18\x04 \x01(\x0b\x32\x12.meta_sink.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xb4\x01\n\x1bMetaSinkWorkerConfiguration\x12#\n\x05input\x18\x01 \x01(\x0b\x32\x14.meta_sink.SinkInput\x12\x1d\n\x15\x61\x63\x63\x65ssTokenDependency\x18\x02 \x01(\t\x12\x13\n\x0b\x61\x64\x41\x63\x63ountId\x18\x03 \x01(\t\x12\x14\n\x0c\x61udienceName\x18\x04 \x01(\t\x12\x17\n\napiVersion\x18\x05 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_apiVersionb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meta_sink_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SINGLEFILE']._serialized_start=30
  _globals['_SINGLEFILE']._serialized_end=56
  _globals['_RAWFILE']._serialized_start=58
  _globals['_RAWFILE']._serialized_end=67
  _globals['_ZIPFILE']._serialized_start=69
  _globals['_ZIPFILE']._serialized_end=136
  _globals['_SINKINPUT']._serialized_start=138
  _globals['_SINKINPUT']._serialized_end=261
  _globals['_METASINKWORKERCONFIGURATION']._serialized_start=264
  _globals['_METASINKWORKERCONFIGURATION']._serialized_end=444
# @@protoc_insertion_point(module_scope)
