# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: salesforce.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'salesforce.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10salesforce.proto\x12\nsalesforce\"\x06\n\x04\x43ore\"\x10\n\x0eMarketingCloud\"s\n\nImportRole\x12 \n\x04\x63ore\x18\x01 \x01(\x0b\x32\x10.salesforce.CoreH\x00\x12\x34\n\x0emarketingCloud\x18\x02 \x01(\x0b\x32\x1a.salesforce.MarketingCloudH\x00\x42\r\n\x0bproductType\"2\n\nExportRole\x12$\n\x05input\x18\x01 \x01(\x0b\x32\x15.salesforce.SinkInput\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"D\n\x07ZipFile\x12,\n\nsingleFile\x18\x01 \x01(\x0b\x32\x16.salesforce.SingleFileH\x00\x42\x0b\n\tselection\"}\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\"\n\x03raw\x18\x03 \x01(\x0b\x32\x13.salesforce.RawFileH\x00\x12\"\n\x03zip\x18\x04 \x01(\x0b\x32\x13.salesforce.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xc6\x01\n\x1dSalesforceWorkerConfiguration\x12\x1d\n\x15\x63redentialsDependency\x18\x01 \x01(\t\x12,\n\nimportRole\x18\x02 \x01(\x0b\x32\x16.salesforce.ImportRoleH\x00\x12,\n\nexportRole\x18\x03 \x01(\x0b\x32\x16.salesforce.ExportRoleH\x00\x12\x11\n\tdomainUrl\x18\x04 \x01(\t\x12\x0f\n\x07\x61piName\x18\x05 \x01(\tB\x06\n\x04roleb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'salesforce_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_CORE']._serialized_start=32
  _globals['_CORE']._serialized_end=38
  _globals['_MARKETINGCLOUD']._serialized_start=40
  _globals['_MARKETINGCLOUD']._serialized_end=56
  _globals['_IMPORTROLE']._serialized_start=58
  _globals['_IMPORTROLE']._serialized_end=173
  _globals['_EXPORTROLE']._serialized_start=175
  _globals['_EXPORTROLE']._serialized_end=225
  _globals['_SINGLEFILE']._serialized_start=227
  _globals['_SINGLEFILE']._serialized_end=253
  _globals['_RAWFILE']._serialized_start=255
  _globals['_RAWFILE']._serialized_end=264
  _globals['_ZIPFILE']._serialized_start=266
  _globals['_ZIPFILE']._serialized_end=334
  _globals['_SINKINPUT']._serialized_start=336
  _globals['_SINKINPUT']._serialized_end=461
  _globals['_SALESFORCEWORKERCONFIGURATION']._serialized_start=464
  _globals['_SALESFORCEWORKERCONFIGURATION']._serialized_end=662
# @@protoc_insertion_point(module_scope)
