# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gcg.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'gcg.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import data_room_pb2 as data__room__pb2
from . import identity_endorsement_pb2 as identity__endorsement__pb2
from . import release_policy_pb2 as release__policy__pb2
from . import secret_store_pb2 as secret__store__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tgcg.proto\x12\x03gcg\x1a\x0f\x64\x61ta_room.proto\x1a\x1aidentity_endorsement.proto\x1a\x14release_policy.proto\x1a\x12secret_store.proto\"\x82\x0e\n\nGcgRequest\x12\x1f\n\x08userAuth\x18\x01 \x01(\x0b\x32\r.gcg.UserAuth\x12;\n\x15\x63reateDataRoomRequest\x18\x02 \x01(\x0b\x32\x1a.gcg.CreateDataRoomRequestH\x00\x12?\n\x17retrieveDataRoomRequest\x18\x03 \x01(\x0b\x32\x1c.gcg.RetrieveDataRoomRequestH\x00\x12g\n+retrieveCurrentDataRoomConfigurationRequest\x18\x04 \x01(\x0b\x32\x30.gcg.RetrieveCurrentDataRoomConfigurationRequestH\x00\x12K\n\x1dretrieveDataRoomStatusRequest\x18\x05 \x01(\x0b\x32\".gcg.RetrieveDataRoomStatusRequestH\x00\x12G\n\x1bupdateDataRoomStatusRequest\x18\x06 \x01(\x0b\x32 .gcg.UpdateDataRoomStatusRequestH\x00\x12?\n\x17retrieveAuditLogRequest\x18\x07 \x01(\x0b\x32\x1c.gcg.RetrieveAuditLogRequestH\x00\x12O\n\x1fpublishDatasetToDataRoomRequest\x18\x08 \x01(\x0b\x32$.gcg.PublishDatasetToDataRoomRequestH\x00\x12Q\n retrievePublishedDatasetsRequest\x18\t \x01(\x0b\x32%.gcg.RetrievePublishedDatasetsRequestH\x00\x12K\n\x1dremovePublishedDatasetRequest\x18\n \x01(\x0b\x32\".gcg.RemovePublishedDatasetRequestH\x00\x12;\n\x15\x65xecuteComputeRequest\x18\x0b \x01(\x0b\x32\x1a.gcg.ExecuteComputeRequestH\x00\x12\x31\n\x10jobStatusRequest\x18\x0c \x01(\x0b\x32\x15.gcg.JobStatusRequestH\x00\x12\x33\n\x11getResultsRequest\x18\r \x01(\x0b\x32\x16.gcg.GetResultsRequestH\x00\x12Q\n createConfigurationCommitRequest\x18\x0e \x01(\x0b\x32%.gcg.CreateConfigurationCommitRequestH\x00\x12U\n\"retrieveConfigurationCommitRequest\x18\x0f \x01(\x0b\x32\'.gcg.RetrieveConfigurationCommitRequestH\x00\x12Q\n executeDevelopmentComputeRequest\x18\x10 \x01(\x0b\x32%.gcg.ExecuteDevelopmentComputeRequestH\x00\x12[\n%generateMergeApprovalSignatureRequest\x18\x11 \x01(\x0b\x32*.gcg.GenerateMergeApprovalSignatureRequestH\x00\x12O\n\x1fmergeConfigurationCommitRequest\x18\x12 \x01(\x0b\x32$.gcg.MergeConfigurationCommitRequestH\x00\x12g\n+retrieveConfigurationCommitApproversRequest\x18\x13 \x01(\x0b\x32\x30.gcg.RetrieveConfigurationCommitApproversRequestH\x00\x12\x41\n\x18\x63\x61sAuxiliaryStateRequest\x18\x14 \x01(\x0b\x32\x1d.gcg.CasAuxiliaryStateRequestH\x00\x12\x43\n\x19readAuxiliaryStateRequest\x18\x15 \x01(\x0b\x32\x1e.gcg.ReadAuxiliaryStateRequestH\x00\x12O\n\x1fretrieveUsedAirlockQuotaRequest\x18\x16 \x01(\x0b\x32$.gcg.RetrieveUsedAirlockQuotaRequestH\x00\x12;\n\x15getResultsSizeRequest\x18\x17 \x01(\x0b\x32\x1a.gcg.GetResultsSizeRequestH\x00\x12\x46\n\x12\x65ndorsementRequest\x18\x65 \x01(\x0b\x32(.identity_endorsement.EndorsementRequestH\x00\x42\r\n\x0bgcg_request\"\x9c\x0e\n\x0bGcgResponse\x12\x11\n\x07\x66\x61ilure\x18\x01 \x01(\tH\x00\x12=\n\x16\x63reateDataRoomResponse\x18\x02 \x01(\x0b\x32\x1b.gcg.CreateDataRoomResponseH\x00\x12\x41\n\x18retrieveDataRoomResponse\x18\x03 \x01(\x0b\x32\x1d.gcg.RetrieveDataRoomResponseH\x00\x12i\n,retrieveCurrentDataRoomConfigurationResponse\x18\x04 \x01(\x0b\x32\x31.gcg.RetrieveCurrentDataRoomConfigurationResponseH\x00\x12M\n\x1eretrieveDataRoomStatusResponse\x18\x05 \x01(\x0b\x32#.gcg.RetrieveDataRoomStatusResponseH\x00\x12I\n\x1cupdateDataRoomStatusResponse\x18\x06 \x01(\x0b\x32!.gcg.UpdateDataRoomStatusResponseH\x00\x12\x41\n\x18retrieveAuditLogResponse\x18\x07 \x01(\x0b\x32\x1d.gcg.RetrieveAuditLogResponseH\x00\x12Q\n publishDatasetToDataRoomResponse\x18\x08 \x01(\x0b\x32%.gcg.PublishDatasetToDataRoomResponseH\x00\x12S\n!retrievePublishedDatasetsResponse\x18\t \x01(\x0b\x32&.gcg.RetrievePublishedDatasetsResponseH\x00\x12M\n\x1eremovePublishedDatasetResponse\x18\n \x01(\x0b\x32#.gcg.RemovePublishedDatasetResponseH\x00\x12=\n\x16\x65xecuteComputeResponse\x18\x0b \x01(\x0b\x32\x1b.gcg.ExecuteComputeResponseH\x00\x12\x33\n\x11jobStatusResponse\x18\x0c \x01(\x0b\x32\x16.gcg.JobStatusResponseH\x00\x12?\n\x17getResultsResponseChunk\x18\r \x01(\x0b\x32\x1c.gcg.GetResultsResponseChunkH\x00\x12\x41\n\x18getResultsResponseFooter\x18\x0e \x01(\x0b\x32\x1d.gcg.GetResultsResponseFooterH\x00\x12S\n!createConfigurationCommitResponse\x18\x0f \x01(\x0b\x32&.gcg.CreateConfigurationCommitResponseH\x00\x12W\n#retrieveConfigurationCommitResponse\x18\x10 \x01(\x0b\x32(.gcg.RetrieveConfigurationCommitResponseH\x00\x12]\n&generateMergeApprovalSignatureResponse\x18\x11 \x01(\x0b\x32+.gcg.GenerateMergeApprovalSignatureResponseH\x00\x12Q\n mergeConfigurationCommitResponse\x18\x12 \x01(\x0b\x32%.gcg.MergeConfigurationCommitResponseH\x00\x12i\n,retrieveConfigurationCommitApproversResponse\x18\x13 \x01(\x0b\x32\x31.gcg.RetrieveConfigurationCommitApproversResponseH\x00\x12\x43\n\x19\x63\x61sAuxiliaryStateResponse\x18\x14 \x01(\x0b\x32\x1e.gcg.CasAuxiliaryStateResponseH\x00\x12\x45\n\x1areadAuxiliaryStateResponse\x18\x15 \x01(\x0b\x32\x1f.gcg.ReadAuxiliaryStateResponseH\x00\x12Q\n retrieveUsedAirlockQuotaResponse\x18\x16 \x01(\x0b\x32%.gcg.RetrieveUsedAirlockQuotaResponseH\x00\x12=\n\x16getResultsSizeResponse\x18\x17 \x01(\x0b\x32\x1b.gcg.GetResultsSizeResponseH\x00\x12H\n\x13\x65ndorsementResponse\x18\x65 \x01(\x0b\x32).identity_endorsement.EndorsementResponseH\x00\x42\x0e\n\x0cgcg_response\"i\n\x08UserAuth\x12\x15\n\x03pki\x18\x01 \x01(\x0b\x32\x08.gcg.Pki\x12\x46\n\x13\x65nclaveEndorsements\x18\x02 \x01(\x0b\x32).identity_endorsement.EnclaveEndorsements\"=\n\x03Pki\x12\x14\n\x0c\x63\x65rtChainPem\x18\x01 \x01(\x0c\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x12\r\n\x05idMac\x18\x03 \x01(\x0c\"\xb4\x01\n\x15\x43reateDataRoomRequest\x12%\n\x08\x64\x61taRoom\x18\x01 \x01(\x0b\x32\x13.data_room.DataRoom\x12$\n\x17highLevelRepresentation\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x12\x1d\n\x10\x64\x61taRoomMetadata\x18\x03 \x01(\x0cH\x01\x88\x01\x01\x42\x1a\n\x18_highLevelRepresentationB\x13\n\x11_dataRoomMetadata\"\x8c\x01\n\x16\x43reateDataRoomResponse\x12\x14\n\ndataRoomId\x18\x01 \x01(\x0cH\x00\x12?\n\x17\x64\x61taRoomValidationError\x18\x02 \x01(\x0b\x32\x1c.gcg.DataRoomValidationErrorH\x00\x42\x1b\n\x19\x63reate_data_room_response\"\xc6\x02\n\x17\x44\x61taRoomValidationError\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x1c\n\x0fpermissionIndex\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x1a\n\rcomputeNodeId\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10userPermissionId\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\'\n\x1a\x61ttestationSpecificationId\x18\x05 \x01(\tH\x03\x88\x01\x01\x12#\n\x16\x61uthenticationMethodId\x18\x06 \x01(\tH\x04\x88\x01\x01\x42\x12\n\x10_permissionIndexB\x10\n\x0e_computeNodeIdB\x13\n\x11_userPermissionIdB\x1d\n\x1b_attestationSpecificationIdB\x19\n\x17_authenticationMethodId\"\x80\x01\n\x1fPublishDatasetToDataRoomRequest\x12\x13\n\x0b\x64\x61tasetHash\x18\x01 \x01(\x0c\x12\x12\n\ndataRoomId\x18\x02 \x01(\x0c\x12\x0e\n\x06leafId\x18\x03 \x01(\t\x12\x15\n\rencryptionKey\x18\x04 \x01(\x0c\x12\r\n\x05scope\x18\x05 \x01(\x0c\"\"\n PublishDatasetToDataRoomResponse\"\xe2\x02\n\x15\x45xecuteComputeRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\x16\n\x0e\x63omputeNodeIds\x18\x02 \x03(\t\x12\x10\n\x08isDryRun\x18\x03 \x01(\x08\x12\r\n\x05scope\x18\x04 \x01(\x0c\x12>\n\nparameters\x18\x05 \x03(\x0b\x32*.gcg.ExecuteComputeRequest.ParametersEntry\x12\x42\n\x0ctestDatasets\x18\x06 \x03(\x0b\x32,.gcg.ExecuteComputeRequest.TestDatasetsEntry\x1a\x31\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x45\n\x11TestDatasetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.gcg.TestDataset:\x02\x38\x01\"\x8e\x03\n ExecuteDevelopmentComputeRequest\x12\x1d\n\x15\x63onfigurationCommitId\x18\x01 \x01(\x0c\x12\x16\n\x0e\x63omputeNodeIds\x18\x02 \x03(\t\x12\x10\n\x08isDryRun\x18\x03 \x01(\x08\x12\r\n\x05scope\x18\x04 \x01(\x0c\x12I\n\nparameters\x18\x05 \x03(\x0b\x32\x35.gcg.ExecuteDevelopmentComputeRequest.ParametersEntry\x12M\n\x0ctestDatasets\x18\x06 \x03(\x0b\x32\x37.gcg.ExecuteDevelopmentComputeRequest.TestDatasetsEntry\x1a\x31\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x45\n\x11TestDatasetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1f\n\x05value\x18\x02 \x01(\x0b\x32\x10.gcg.TestDataset:\x02\x38\x01\":\n\x0bTestDataset\x12\x15\n\rencryptionKey\x18\x01 \x01(\x0c\x12\x14\n\x0cmanifestHash\x18\x02 \x01(\x0c\"\'\n\x16\x45xecuteComputeResponse\x12\r\n\x05jobId\x18\x01 \x01(\x0c\"!\n\x10JobStatusRequest\x12\r\n\x05jobId\x18\x01 \x01(\x0c\"3\n\x11JobStatusResponse\x12\x1e\n\x16\x63ompleteComputeNodeIds\x18\x01 \x03(\t\"9\n\x11GetResultsRequest\x12\r\n\x05jobId\x18\x01 \x01(\x0c\x12\x15\n\rcomputeNodeId\x18\x02 \x01(\t\"=\n\x15GetResultsSizeRequest\x12\r\n\x05jobId\x18\x01 \x01(\x0c\x12\x15\n\rcomputeNodeId\x18\x02 \x01(\t\"\'\n\x17GetResultsResponseChunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\x1a\n\x18GetResultsResponseFooter\"-\n\x17RetrieveDataRoomRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"\xb4\x01\n\x18RetrieveDataRoomResponse\x12%\n\x08\x64\x61taRoom\x18\x01 \x01(\x0b\x32\x13.data_room.DataRoom\x12/\n\x07\x63ommits\x18\x02 \x03(\x0b\x32\x1e.data_room.ConfigurationCommit\x12$\n\x17highLevelRepresentation\x18\x03 \x01(\x0cH\x00\x88\x01\x01\x42\x1a\n\x18_highLevelRepresentation\"-\n\x17RetrieveAuditLogRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"\'\n\x18RetrieveAuditLogResponse\x12\x0b\n\x03log\x18\x01 \x01(\x0c\"3\n\x1dRetrieveDataRoomStatusRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"E\n\x1eRetrieveDataRoomStatusResponse\x12#\n\x06status\x18\x01 \x01(\x0e\x32\x13.gcg.DataRoomStatus\"V\n\x1bUpdateDataRoomStatusRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12#\n\x06status\x18\x02 \x01(\x0e\x32\x13.gcg.DataRoomStatus\"\x1e\n\x1cUpdateDataRoomStatusResponse\"6\n RetrievePublishedDatasetsRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"X\n\x10PublishedDataset\x12\x0e\n\x06leafId\x18\x01 \x01(\t\x12\x0c\n\x04user\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\x04\x12\x13\n\x0b\x64\x61tasetHash\x18\x04 \x01(\x0c\"U\n!RetrievePublishedDatasetsResponse\x12\x30\n\x11publishedDatasets\x18\x01 \x03(\x0b\x32\x15.gcg.PublishedDataset\"C\n\x1dRemovePublishedDatasetRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\x0e\n\x06leafId\x18\x02 \x01(\t\" \n\x1eRemovePublishedDatasetResponse\"\x94\x01\n CreateConfigurationCommitRequest\x12.\n\x06\x63ommit\x18\x01 \x01(\x0b\x32\x1e.data_room.ConfigurationCommit\x12$\n\x17highLevelRepresentation\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x1a\n\x18_highLevelRepresentation\"5\n!CreateConfigurationCommitResponse\x12\x10\n\x08\x63ommitId\x18\x01 \x01(\x0c\"9\n%GenerateMergeApprovalSignatureRequest\x12\x10\n\x08\x63ommitId\x18\x01 \x01(\x0c\";\n&GenerateMergeApprovalSignatureResponse\x12\x11\n\tsignature\x18\x01 \x01(\x0c\"\xa0\x02\n\x1fMergeConfigurationCommitRequest\x12\x10\n\x08\x63ommitId\x18\x01 \x01(\x0c\x12X\n\x12\x61pprovalSignatures\x18\x02 \x03(\x0b\x32<.gcg.MergeConfigurationCommitRequest.ApprovalSignaturesEntry\x12/\n\"newDataRoomHighLevelRepresentation\x18\x03 \x01(\x0cH\x00\x88\x01\x01\x1a\x39\n\x17\x41pprovalSignaturesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c:\x02\x38\x01\x42%\n#_newDataRoomHighLevelRepresentation\"A\n+RetrieveCurrentDataRoomConfigurationRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"t\n,RetrieveCurrentDataRoomConfigurationResponse\x12\x37\n\rconfiguration\x18\x01 \x01(\x0b\x32 .data_room.DataRoomConfiguration\x12\x0b\n\x03pin\x18\x02 \x01(\x0c\"?\n+RetrieveConfigurationCommitApproversRequest\x12\x10\n\x08\x63ommitId\x18\x01 \x01(\x0c\"A\n,RetrieveConfigurationCommitApproversResponse\x12\x11\n\tapprovers\x18\x01 \x03(\t\"6\n\"RetrieveConfigurationCommitRequest\x12\x10\n\x08\x63ommitId\x18\x01 \x01(\x0c\"\x97\x01\n#RetrieveConfigurationCommitResponse\x12.\n\x06\x63ommit\x18\x01 \x01(\x0b\x32\x1e.data_room.ConfigurationCommit\x12$\n\x17highLevelRepresentation\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x1a\n\x18_highLevelRepresentation\"[\n\x18\x43\x61sAuxiliaryStateRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\x12\r\n\x05index\x18\x02 \x01(\x04\x12\x12\n\x05value\x18\x03 \x01(\x0cH\x00\x88\x01\x01\x42\x08\n\x06_value\"Y\n\x19\x43\x61sAuxiliaryStateResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\r\n\x05index\x18\x02 \x01(\x04\x12\x12\n\x05value\x18\x03 \x01(\x0cH\x00\x88\x01\x01\x42\x08\n\x06_value\"/\n\x19ReadAuxiliaryStateRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"F\n\x1aReadAuxiliaryStateResponse\x12(\n\x06values\x18\x02 \x03(\x0b\x32\x18.gcg.AuxiliaryStateValue\"A\n\x13\x41uxiliaryStateValue\x12\x0c\n\x04user\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x04\x12\r\n\x05value\x18\x03 \x01(\x0c\"\"\n MergeConfigurationCommitResponse\"|\n\x10\x44riverTaskConfig\x12\x1f\n\x04noop\x18\x01 \x01(\x0b\x32\x0f.gcg.NoopConfigH\x00\x12\x31\n\rstaticContent\x18\x02 \x01(\x0b\x32\x18.gcg.StaticContentConfigH\x00\x42\x14\n\x12\x64river_task_config\"\x0c\n\nNoopConfig\"&\n\x13StaticContentConfig\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\"5\n\x1fRetrieveUsedAirlockQuotaRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\"P\n RetrieveUsedAirlockQuotaResponse\x12,\n\rairlockQuotas\x18\x01 \x03(\x0b\x32\x15.gcg.AirlockQuotaInfo\"U\n\x10\x41irlockQuotaInfo\x12\x15\n\rairlockNodeId\x18\x01 \x01(\t\x12\x12\n\nquotaBytes\x18\x02 \x01(\x04\x12\x16\n\x0eusedQuotaBytes\x18\x03 \x01(\x04\"+\n\x16GetResultsSizeResponse\x12\x11\n\tsizeBytes\x18\x01 \x01(\x04\"\x85\x01\n\x0cGcgRequestV2\x12\x36\n\x0funauthenticated\x18\x01 \x01(\x0b\x32\x1b.gcg.UnauthenticatedRequestH\x00\x12\x32\n\rauthenticated\x18\x02 \x01(\x0b\x32\x19.gcg.AuthenticatedRequestH\x00\x42\t\n\x07request\"\xe0\x04\n\x14\x41uthenticatedRequest\x12\x15\n\x0b\x61\x63\x63\x65ssToken\x18\x01 \x01(\tH\x00\x12\x12\n\x08\x61piToken\x18\x02 \x01(\tH\x00\x12\x30\n\x0c\x63reatePolicy\x18\x03 \x01(\x0b\x32\x18.gcg.CreatePolicyRequestH\x01\x12;\n\x0bsecretStore\x18\x04 \x01(\x0b\x32$.secret_store_api.SecretStoreRequestH\x01\x12<\n\x12getVerificationKey\x18\x06 \x01(\x0b\x32\x1e.gcg.GetVerificationKeyRequestH\x01\x12*\n\tdcrAction\x18\x07 \x01(\x0b\x32\x15.gcg.DcrActionRequestH\x01\x12\x34\n\x0e\x63reateApiToken\x18\x08 \x01(\x0b\x32\x1a.gcg.CreateApiTokenRequestH\x01\x12\x30\n\x0cgetApiTokens\x18\t \x01(\x0b\x32\x18.gcg.GetApiTokensRequestH\x01\x12\x34\n\x0e\x64\x65leteApiToken\x18\n \x01(\x0b\x32\x1a.gcg.DeleteApiTokenRequestH\x01\x12N\n\x1bretrieveResultEncryptionKey\x18\x0b \x01(\x0b\x32\'.gcg.RetrieveResultEncryptionKeyRequestH\x01\x12\x42\n\x15\x65xportResultAsDataset\x18\x0c \x01(\x0b\x32!.gcg.ExportResultAsDatasetRequestH\x01\x42\x07\n\x05tokenB\t\n\x07request\"\xb7\x04\n\x15\x41uthenticatedResponse\x12\x31\n\x0c\x63reatePolicy\x18\x03 \x01(\x0b\x32\x19.gcg.CreatePolicyResponseH\x00\x12<\n\x0bsecretStore\x18\x04 \x01(\x0b\x32%.secret_store_api.SecretStoreResponseH\x00\x12=\n\x12getVerificationKey\x18\x06 \x01(\x0b\x32\x1f.gcg.GetVerificationKeyResponseH\x00\x12+\n\tdcrAction\x18\x07 \x01(\x0b\x32\x16.gcg.DcrActionResponseH\x00\x12\x35\n\x0e\x63reateApiToken\x18\x08 \x01(\x0b\x32\x1b.gcg.CreateApiTokenResponseH\x00\x12\x31\n\x0cgetApiTokens\x18\t \x01(\x0b\x32\x19.gcg.GetApiTokensResponseH\x00\x12\x35\n\x0e\x64\x65leteApiToken\x18\n \x01(\x0b\x32\x1b.gcg.DeleteApiTokenResponseH\x00\x12O\n\x1bretrieveResultEncryptionKey\x18\x0b \x01(\x0b\x32(.gcg.RetrieveResultEncryptionKeyResponseH\x00\x12\x43\n\x15\x65xportResultAsDataset\x18\x0c \x01(\x0b\x32\".gcg.ExportResultAsDatasetResponseH\x00\x42\n\n\x08response\"Y\n\x16UnauthenticatedRequest\x12\x34\n\x0e\x61uthentication\x18\x01 \x01(\x0b\x32\x1a.gcg.AuthenticationRequestH\x00\x42\t\n\x07request\"_\n\rGcgResponseV2\x12/\n\x07success\x18\x01 \x01(\x0b\x32\x1c.gcg.GcgSuccessfulResponseV2H\x00\x12\x11\n\x07\x66\x61ilure\x18\x02 \x01(\tH\x00\x42\n\n\x08response\"\x9c\x01\n\x15\x41uthenticationRequest\x12:\n\nemailStart\x18\x02 \x01(\x0b\x32$.gcg.EmailStartAuthenticationRequestH\x00\x12<\n\x0b\x65mailFinish\x18\x03 \x01(\x0b\x32%.gcg.EmailFinishAuthenticationRequestH\x00\x42\t\n\x07request\"\xa0\x01\n\x16\x41uthenticationResponse\x12;\n\nemailStart\x18\x02 \x01(\x0b\x32%.gcg.EmailStartAuthenticationResponseH\x00\x12=\n\x0b\x65mailFinish\x18\x03 \x01(\x0b\x32&.gcg.EmailFinishAuthenticationResponseH\x00\x42\n\n\x08response\"0\n\x1f\x45mailStartAuthenticationRequest\x12\r\n\x05\x65mail\x18\x01 \x01(\t\"9\n EmailStartAuthenticationResponse\x12\x15\n\rcorrelationId\x18\x01 \x01(\t\"?\n EmailFinishAuthenticationRequest\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\"\x83\x02\n!EmailFinishAuthenticationResponse\x12L\n\rincorrectCode\x18\x01 \x01(\x0b\x32\x33.gcg.EmailFinishAuthenticationResponseIncorrectCodeH\x00\x12\x42\n\x08notFound\x18\x02 \x01(\x0b\x32..gcg.EmailFinishAuthenticationResponseNotFoundH\x00\x12@\n\x07success\x18\x03 \x01(\x0b\x32-.gcg.EmailFinishAuthenticationResponseSuccessH\x00\x42\n\n\x08response\"0\n.EmailFinishAuthenticationResponseIncorrectCode\"+\n)EmailFinishAuthenticationResponseNotFound\"9\n(EmailFinishAuthenticationResponseSuccess\x12\r\n\x05token\x18\x01 \x01(\t\"\x93\x01\n\x17GcgSuccessfulResponseV2\x12\x37\n\x0funauthenticated\x18\x01 \x01(\x0b\x32\x1c.gcg.UnauthenticatedResponseH\x00\x12\x33\n\rauthenticated\x18\x02 \x01(\x0b\x32\x1a.gcg.AuthenticatedResponseH\x00\x42\n\n\x08response\"\\\n\x17UnauthenticatedResponse\x12\x35\n\x0e\x61uthentication\x18\x01 \x01(\x0b\x32\x1b.gcg.AuthenticationResponseH\x00\x42\n\n\x08response\"\x1b\n\x19GetVerificationKeyRequest\"5\n\x1aGetVerificationKeyResponse\x12\x17\n\x0fverificationKey\x18\x01 \x01(\x0c\"f\n\x13\x43reatePolicyRequest\x12\x31\n\x06policy\x18\x01 \x01(\x0b\x32!.release_policy_api.ReleasePolicy\x12\x12\n\x05scope\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x08\n\x06_scope\"(\n\x14\x43reatePolicyResponse\x12\x10\n\x08policyId\x18\x01 \x01(\t\"6\n\x10\x44\x63rActionRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\x0c\"%\n\x11\x44\x63rActionResponse\x12\x10\n\x08response\x18\x01 \x01(\x0c\"%\n\x15\x43reateApiTokenRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\'\n\x16\x43reateApiTokenResponse\x12\r\n\x05token\x18\x01 \x01(\t\"\x15\n\x13GetApiTokensRequest\">\n\x14GetApiTokensResponse\x12&\n\x06tokens\x18\x01 \x03(\x0b\x32\x16.gcg.GetApiTokenResult\"0\n\x11GetApiTokenResult\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\t\"&\n\x15\x44\x65leteApiTokenRequest\x12\r\n\x05token\x18\x01 \x01(\t\"\x18\n\x16\x44\x65leteApiTokenResponse\"N\n\"RetrieveResultEncryptionKeyRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x19\n\x11taskResultHashHex\x18\x02 \x01(\t\"U\n#RetrieveResultEncryptionKeyResponse\x12\x17\n\x0fmanifestHashHex\x18\x01 \x01(\t\x12\x15\n\rencryptionKey\x18\x02 \x01(\x0c\"\xb2\x01\n\x1c\x45xportResultAsDatasetRequest\x12\r\n\x05jobId\x18\x01 \x01(\t\x12\x19\n\x11taskResultHashHex\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x61tasetName\x18\x03 \x01(\t\x12\x1d\n\x03raw\x18\x04 \x01(\x0b\x32\x0e.gcg.RawExportH\x00\x12\x1d\n\x03zip\x18\x05 \x01(\x0b\x32\x0e.gcg.ZipExportH\x00\x42\x15\n\x13\x65xportSpecification\"\x0b\n\tRawExport\"\x19\n\tZipExport\x12\x0c\n\x04path\x18\x01 \x01(\t\"]\n\x1d\x45xportResultAsDatasetResponse\x12\x17\n\x0fmanifestHashHex\x18\x01 \x01(\t\x12\x11\n\tdatasetId\x18\x02 \x01(\t\x12\x10\n\x08secretId\x18\x03 \x01(\t*)\n\x0e\x44\x61taRoomStatus\x12\n\n\x06\x41\x63tive\x10\x00\x12\x0b\n\x07Stopped\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gcg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_EXECUTECOMPUTEREQUEST_PARAMETERSENTRY']._loaded_options = None
  _globals['_EXECUTECOMPUTEREQUEST_PARAMETERSENTRY']._serialized_options = b'8\001'
  _globals['_EXECUTECOMPUTEREQUEST_TESTDATASETSENTRY']._loaded_options = None
  _globals['_EXECUTECOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_options = b'8\001'
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_PARAMETERSENTRY']._loaded_options = None
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_PARAMETERSENTRY']._serialized_options = b'8\001'
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_TESTDATASETSENTRY']._loaded_options = None
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_options = b'8\001'
  _globals['_MERGECONFIGURATIONCOMMITREQUEST_APPROVALSIGNATURESENTRY']._loaded_options = None
  _globals['_MERGECONFIGURATIONCOMMITREQUEST_APPROVALSIGNATURESENTRY']._serialized_options = b'8\001'
  _globals['_DATAROOMSTATUS']._serialized_start=12506
  _globals['_DATAROOMSTATUS']._serialized_end=12547
  _globals['_GCGREQUEST']._serialized_start=106
  _globals['_GCGREQUEST']._serialized_end=1900
  _globals['_GCGRESPONSE']._serialized_start=1903
  _globals['_GCGRESPONSE']._serialized_end=3723
  _globals['_USERAUTH']._serialized_start=3725
  _globals['_USERAUTH']._serialized_end=3830
  _globals['_PKI']._serialized_start=3832
  _globals['_PKI']._serialized_end=3893
  _globals['_CREATEDATAROOMREQUEST']._serialized_start=3896
  _globals['_CREATEDATAROOMREQUEST']._serialized_end=4076
  _globals['_CREATEDATAROOMRESPONSE']._serialized_start=4079
  _globals['_CREATEDATAROOMRESPONSE']._serialized_end=4219
  _globals['_DATAROOMVALIDATIONERROR']._serialized_start=4222
  _globals['_DATAROOMVALIDATIONERROR']._serialized_end=4548
  _globals['_PUBLISHDATASETTODATAROOMREQUEST']._serialized_start=4551
  _globals['_PUBLISHDATASETTODATAROOMREQUEST']._serialized_end=4679
  _globals['_PUBLISHDATASETTODATAROOMRESPONSE']._serialized_start=4681
  _globals['_PUBLISHDATASETTODATAROOMRESPONSE']._serialized_end=4715
  _globals['_EXECUTECOMPUTEREQUEST']._serialized_start=4718
  _globals['_EXECUTECOMPUTEREQUEST']._serialized_end=5072
  _globals['_EXECUTECOMPUTEREQUEST_PARAMETERSENTRY']._serialized_start=4952
  _globals['_EXECUTECOMPUTEREQUEST_PARAMETERSENTRY']._serialized_end=5001
  _globals['_EXECUTECOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_start=5003
  _globals['_EXECUTECOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_end=5072
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST']._serialized_start=5075
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST']._serialized_end=5473
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_PARAMETERSENTRY']._serialized_start=4952
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_PARAMETERSENTRY']._serialized_end=5001
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_start=5003
  _globals['_EXECUTEDEVELOPMENTCOMPUTEREQUEST_TESTDATASETSENTRY']._serialized_end=5072
  _globals['_TESTDATASET']._serialized_start=5475
  _globals['_TESTDATASET']._serialized_end=5533
  _globals['_EXECUTECOMPUTERESPONSE']._serialized_start=5535
  _globals['_EXECUTECOMPUTERESPONSE']._serialized_end=5574
  _globals['_JOBSTATUSREQUEST']._serialized_start=5576
  _globals['_JOBSTATUSREQUEST']._serialized_end=5609
  _globals['_JOBSTATUSRESPONSE']._serialized_start=5611
  _globals['_JOBSTATUSRESPONSE']._serialized_end=5662
  _globals['_GETRESULTSREQUEST']._serialized_start=5664
  _globals['_GETRESULTSREQUEST']._serialized_end=5721
  _globals['_GETRESULTSSIZEREQUEST']._serialized_start=5723
  _globals['_GETRESULTSSIZEREQUEST']._serialized_end=5784
  _globals['_GETRESULTSRESPONSECHUNK']._serialized_start=5786
  _globals['_GETRESULTSRESPONSECHUNK']._serialized_end=5825
  _globals['_GETRESULTSRESPONSEFOOTER']._serialized_start=5827
  _globals['_GETRESULTSRESPONSEFOOTER']._serialized_end=5853
  _globals['_RETRIEVEDATAROOMREQUEST']._serialized_start=5855
  _globals['_RETRIEVEDATAROOMREQUEST']._serialized_end=5900
  _globals['_RETRIEVEDATAROOMRESPONSE']._serialized_start=5903
  _globals['_RETRIEVEDATAROOMRESPONSE']._serialized_end=6083
  _globals['_RETRIEVEAUDITLOGREQUEST']._serialized_start=6085
  _globals['_RETRIEVEAUDITLOGREQUEST']._serialized_end=6130
  _globals['_RETRIEVEAUDITLOGRESPONSE']._serialized_start=6132
  _globals['_RETRIEVEAUDITLOGRESPONSE']._serialized_end=6171
  _globals['_RETRIEVEDATAROOMSTATUSREQUEST']._serialized_start=6173
  _globals['_RETRIEVEDATAROOMSTATUSREQUEST']._serialized_end=6224
  _globals['_RETRIEVEDATAROOMSTATUSRESPONSE']._serialized_start=6226
  _globals['_RETRIEVEDATAROOMSTATUSRESPONSE']._serialized_end=6295
  _globals['_UPDATEDATAROOMSTATUSREQUEST']._serialized_start=6297
  _globals['_UPDATEDATAROOMSTATUSREQUEST']._serialized_end=6383
  _globals['_UPDATEDATAROOMSTATUSRESPONSE']._serialized_start=6385
  _globals['_UPDATEDATAROOMSTATUSRESPONSE']._serialized_end=6415
  _globals['_RETRIEVEPUBLISHEDDATASETSREQUEST']._serialized_start=6417
  _globals['_RETRIEVEPUBLISHEDDATASETSREQUEST']._serialized_end=6471
  _globals['_PUBLISHEDDATASET']._serialized_start=6473
  _globals['_PUBLISHEDDATASET']._serialized_end=6561
  _globals['_RETRIEVEPUBLISHEDDATASETSRESPONSE']._serialized_start=6563
  _globals['_RETRIEVEPUBLISHEDDATASETSRESPONSE']._serialized_end=6648
  _globals['_REMOVEPUBLISHEDDATASETREQUEST']._serialized_start=6650
  _globals['_REMOVEPUBLISHEDDATASETREQUEST']._serialized_end=6717
  _globals['_REMOVEPUBLISHEDDATASETRESPONSE']._serialized_start=6719
  _globals['_REMOVEPUBLISHEDDATASETRESPONSE']._serialized_end=6751
  _globals['_CREATECONFIGURATIONCOMMITREQUEST']._serialized_start=6754
  _globals['_CREATECONFIGURATIONCOMMITREQUEST']._serialized_end=6902
  _globals['_CREATECONFIGURATIONCOMMITRESPONSE']._serialized_start=6904
  _globals['_CREATECONFIGURATIONCOMMITRESPONSE']._serialized_end=6957
  _globals['_GENERATEMERGEAPPROVALSIGNATUREREQUEST']._serialized_start=6959
  _globals['_GENERATEMERGEAPPROVALSIGNATUREREQUEST']._serialized_end=7016
  _globals['_GENERATEMERGEAPPROVALSIGNATURERESPONSE']._serialized_start=7018
  _globals['_GENERATEMERGEAPPROVALSIGNATURERESPONSE']._serialized_end=7077
  _globals['_MERGECONFIGURATIONCOMMITREQUEST']._serialized_start=7080
  _globals['_MERGECONFIGURATIONCOMMITREQUEST']._serialized_end=7368
  _globals['_MERGECONFIGURATIONCOMMITREQUEST_APPROVALSIGNATURESENTRY']._serialized_start=7272
  _globals['_MERGECONFIGURATIONCOMMITREQUEST_APPROVALSIGNATURESENTRY']._serialized_end=7329
  _globals['_RETRIEVECURRENTDATAROOMCONFIGURATIONREQUEST']._serialized_start=7370
  _globals['_RETRIEVECURRENTDATAROOMCONFIGURATIONREQUEST']._serialized_end=7435
  _globals['_RETRIEVECURRENTDATAROOMCONFIGURATIONRESPONSE']._serialized_start=7437
  _globals['_RETRIEVECURRENTDATAROOMCONFIGURATIONRESPONSE']._serialized_end=7553
  _globals['_RETRIEVECONFIGURATIONCOMMITAPPROVERSREQUEST']._serialized_start=7555
  _globals['_RETRIEVECONFIGURATIONCOMMITAPPROVERSREQUEST']._serialized_end=7618
  _globals['_RETRIEVECONFIGURATIONCOMMITAPPROVERSRESPONSE']._serialized_start=7620
  _globals['_RETRIEVECONFIGURATIONCOMMITAPPROVERSRESPONSE']._serialized_end=7685
  _globals['_RETRIEVECONFIGURATIONCOMMITREQUEST']._serialized_start=7687
  _globals['_RETRIEVECONFIGURATIONCOMMITREQUEST']._serialized_end=7741
  _globals['_RETRIEVECONFIGURATIONCOMMITRESPONSE']._serialized_start=7744
  _globals['_RETRIEVECONFIGURATIONCOMMITRESPONSE']._serialized_end=7895
  _globals['_CASAUXILIARYSTATEREQUEST']._serialized_start=7897
  _globals['_CASAUXILIARYSTATEREQUEST']._serialized_end=7988
  _globals['_CASAUXILIARYSTATERESPONSE']._serialized_start=7990
  _globals['_CASAUXILIARYSTATERESPONSE']._serialized_end=8079
  _globals['_READAUXILIARYSTATEREQUEST']._serialized_start=8081
  _globals['_READAUXILIARYSTATEREQUEST']._serialized_end=8128
  _globals['_READAUXILIARYSTATERESPONSE']._serialized_start=8130
  _globals['_READAUXILIARYSTATERESPONSE']._serialized_end=8200
  _globals['_AUXILIARYSTATEVALUE']._serialized_start=8202
  _globals['_AUXILIARYSTATEVALUE']._serialized_end=8267
  _globals['_MERGECONFIGURATIONCOMMITRESPONSE']._serialized_start=8269
  _globals['_MERGECONFIGURATIONCOMMITRESPONSE']._serialized_end=8303
  _globals['_DRIVERTASKCONFIG']._serialized_start=8305
  _globals['_DRIVERTASKCONFIG']._serialized_end=8429
  _globals['_NOOPCONFIG']._serialized_start=8431
  _globals['_NOOPCONFIG']._serialized_end=8443
  _globals['_STATICCONTENTCONFIG']._serialized_start=8445
  _globals['_STATICCONTENTCONFIG']._serialized_end=8483
  _globals['_RETRIEVEUSEDAIRLOCKQUOTAREQUEST']._serialized_start=8485
  _globals['_RETRIEVEUSEDAIRLOCKQUOTAREQUEST']._serialized_end=8538
  _globals['_RETRIEVEUSEDAIRLOCKQUOTARESPONSE']._serialized_start=8540
  _globals['_RETRIEVEUSEDAIRLOCKQUOTARESPONSE']._serialized_end=8620
  _globals['_AIRLOCKQUOTAINFO']._serialized_start=8622
  _globals['_AIRLOCKQUOTAINFO']._serialized_end=8707
  _globals['_GETRESULTSSIZERESPONSE']._serialized_start=8709
  _globals['_GETRESULTSSIZERESPONSE']._serialized_end=8752
  _globals['_GCGREQUESTV2']._serialized_start=8755
  _globals['_GCGREQUESTV2']._serialized_end=8888
  _globals['_AUTHENTICATEDREQUEST']._serialized_start=8891
  _globals['_AUTHENTICATEDREQUEST']._serialized_end=9499
  _globals['_AUTHENTICATEDRESPONSE']._serialized_start=9502
  _globals['_AUTHENTICATEDRESPONSE']._serialized_end=10069
  _globals['_UNAUTHENTICATEDREQUEST']._serialized_start=10071
  _globals['_UNAUTHENTICATEDREQUEST']._serialized_end=10160
  _globals['_GCGRESPONSEV2']._serialized_start=10162
  _globals['_GCGRESPONSEV2']._serialized_end=10257
  _globals['_AUTHENTICATIONREQUEST']._serialized_start=10260
  _globals['_AUTHENTICATIONREQUEST']._serialized_end=10416
  _globals['_AUTHENTICATIONRESPONSE']._serialized_start=10419
  _globals['_AUTHENTICATIONRESPONSE']._serialized_end=10579
  _globals['_EMAILSTARTAUTHENTICATIONREQUEST']._serialized_start=10581
  _globals['_EMAILSTARTAUTHENTICATIONREQUEST']._serialized_end=10629
  _globals['_EMAILSTARTAUTHENTICATIONRESPONSE']._serialized_start=10631
  _globals['_EMAILSTARTAUTHENTICATIONRESPONSE']._serialized_end=10688
  _globals['_EMAILFINISHAUTHENTICATIONREQUEST']._serialized_start=10690
  _globals['_EMAILFINISHAUTHENTICATIONREQUEST']._serialized_end=10753
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSE']._serialized_start=10756
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSE']._serialized_end=11015
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSEINCORRECTCODE']._serialized_start=11017
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSEINCORRECTCODE']._serialized_end=11065
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSENOTFOUND']._serialized_start=11067
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSENOTFOUND']._serialized_end=11110
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSESUCCESS']._serialized_start=11112
  _globals['_EMAILFINISHAUTHENTICATIONRESPONSESUCCESS']._serialized_end=11169
  _globals['_GCGSUCCESSFULRESPONSEV2']._serialized_start=11172
  _globals['_GCGSUCCESSFULRESPONSEV2']._serialized_end=11319
  _globals['_UNAUTHENTICATEDRESPONSE']._serialized_start=11321
  _globals['_UNAUTHENTICATEDRESPONSE']._serialized_end=11413
  _globals['_GETVERIFICATIONKEYREQUEST']._serialized_start=11415
  _globals['_GETVERIFICATIONKEYREQUEST']._serialized_end=11442
  _globals['_GETVERIFICATIONKEYRESPONSE']._serialized_start=11444
  _globals['_GETVERIFICATIONKEYRESPONSE']._serialized_end=11497
  _globals['_CREATEPOLICYREQUEST']._serialized_start=11499
  _globals['_CREATEPOLICYREQUEST']._serialized_end=11601
  _globals['_CREATEPOLICYRESPONSE']._serialized_start=11603
  _globals['_CREATEPOLICYRESPONSE']._serialized_end=11643
  _globals['_DCRACTIONREQUEST']._serialized_start=11645
  _globals['_DCRACTIONREQUEST']._serialized_end=11699
  _globals['_DCRACTIONRESPONSE']._serialized_start=11701
  _globals['_DCRACTIONRESPONSE']._serialized_end=11738
  _globals['_CREATEAPITOKENREQUEST']._serialized_start=11740
  _globals['_CREATEAPITOKENREQUEST']._serialized_end=11777
  _globals['_CREATEAPITOKENRESPONSE']._serialized_start=11779
  _globals['_CREATEAPITOKENRESPONSE']._serialized_end=11818
  _globals['_GETAPITOKENSREQUEST']._serialized_start=11820
  _globals['_GETAPITOKENSREQUEST']._serialized_end=11841
  _globals['_GETAPITOKENSRESPONSE']._serialized_start=11843
  _globals['_GETAPITOKENSRESPONSE']._serialized_end=11905
  _globals['_GETAPITOKENRESULT']._serialized_start=11907
  _globals['_GETAPITOKENRESULT']._serialized_end=11955
  _globals['_DELETEAPITOKENREQUEST']._serialized_start=11957
  _globals['_DELETEAPITOKENREQUEST']._serialized_end=11995
  _globals['_DELETEAPITOKENRESPONSE']._serialized_start=11997
  _globals['_DELETEAPITOKENRESPONSE']._serialized_end=12021
  _globals['_RETRIEVERESULTENCRYPTIONKEYREQUEST']._serialized_start=12023
  _globals['_RETRIEVERESULTENCRYPTIONKEYREQUEST']._serialized_end=12101
  _globals['_RETRIEVERESULTENCRYPTIONKEYRESPONSE']._serialized_start=12103
  _globals['_RETRIEVERESULTENCRYPTIONKEYRESPONSE']._serialized_end=12188
  _globals['_EXPORTRESULTASDATASETREQUEST']._serialized_start=12191
  _globals['_EXPORTRESULTASDATASETREQUEST']._serialized_end=12369
  _globals['_RAWEXPORT']._serialized_start=12371
  _globals['_RAWEXPORT']._serialized_end=12382
  _globals['_ZIPEXPORT']._serialized_start=12384
  _globals['_ZIPEXPORT']._serialized_end=12409
  _globals['_EXPORTRESULTASDATASETRESPONSE']._serialized_start=12411
  _globals['_EXPORTRESULTASDATASETRESPONSE']._serialized_end=12504
# @@protoc_insertion_point(module_scope)
