# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: integrity_store.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'integrity_store.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15integrity_store.proto\x12\x13integrity_store_api\"b\n\x15IntegrityStoreRequest\x12>\n\x03\x63\x61s\x18\x01 \x01(\x0b\x32/.integrity_store_api.BatchCompareAndSwapRequestH\x00\x42\t\n\x07request\"e\n\x16IntegrityStoreResponse\x12?\n\x03\x63\x61s\x18\x01 \x01(\x0b\x32\x30.integrity_store_api.BatchCompareAndSwapResponseH\x00\x42\n\n\x08response\"Z\n\x1a\x42\x61tchCompareAndSwapRequest\x12<\n\x08requests\x18\x01 \x03(\x0b\x32*.integrity_store_api.CompareAndSwapRequest\"]\n\x1b\x42\x61tchCompareAndSwapResponse\x12>\n\tresponses\x18\x01 \x03(\x0b\x32+.integrity_store_api.CompareAndSwapResponse\"\xbb\x01\n\x15\x43ompareAndSwapRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x14\n\x0c\x63ontent_type\x18\x02 \x01(\t\x12\x1d\n\x10\x65xpectedCasIndex\x18\x03 \x01(\x04H\x00\x88\x01\x01\x12?\n\x08newValue\x18\x04 \x01(\x0b\x32(.integrity_store_api.IntegrityStoreValueH\x01\x88\x01\x01\x42\x13\n\x11_expectedCasIndexB\x0b\n\t_newValue\"\xb5\x01\n\x16\x43ompareAndSwapResponse\x12\x0f\n\x07swapped\x18\x01 \x01(\x08\x12\x1d\n\x10previousCasIndex\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x44\n\rpreviousValue\x18\x03 \x01(\x0b\x32(.integrity_store_api.IntegrityStoreValueH\x01\x88\x01\x01\x42\x13\n\x11_previousCasIndexB\x10\n\x0e_previousValue\"\xc6\x01\n\x13IntegrityStoreValue\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x1d\n\x10plaintextContent\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x1f\n\x12plaintextSignature\x18\x03 \x01(\x0cH\x01\x88\x01\x01\x12\x1d\n\x10\x65ncryptedContent\x18\x04 \x01(\x0cH\x02\x88\x01\x01\x42\x13\n\x11_plaintextContentB\x15\n\x13_plaintextSignatureB\x13\n\x11_encryptedContentb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'integrity_store_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_INTEGRITYSTOREREQUEST']._serialized_start=46
  _globals['_INTEGRITYSTOREREQUEST']._serialized_end=144
  _globals['_INTEGRITYSTORERESPONSE']._serialized_start=146
  _globals['_INTEGRITYSTORERESPONSE']._serialized_end=247
  _globals['_BATCHCOMPAREANDSWAPREQUEST']._serialized_start=249
  _globals['_BATCHCOMPAREANDSWAPREQUEST']._serialized_end=339
  _globals['_BATCHCOMPAREANDSWAPRESPONSE']._serialized_start=341
  _globals['_BATCHCOMPAREANDSWAPRESPONSE']._serialized_end=434
  _globals['_COMPAREANDSWAPREQUEST']._serialized_start=437
  _globals['_COMPAREANDSWAPREQUEST']._serialized_end=624
  _globals['_COMPAREANDSWAPRESPONSE']._serialized_start=627
  _globals['_COMPAREANDSWAPRESPONSE']._serialized_end=808
  _globals['_INTEGRITYSTOREVALUE']._serialized_start=811
  _globals['_INTEGRITYSTOREVALUE']._serialized_end=1009
# @@protoc_insertion_point(module_scope)
