# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: archv2.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import data_room_pb2 as data__room__pb2
from . import release_policy_pb2 as release__policy__pb2
from . import secret_store_pb2 as secret__store__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x61rchv2.proto\x12\x06\x61rchv2\x1a\x0f\x64\x61ta_room.proto\x1a\x14release_policy.proto\x1a\x12secret_store.proto\"\x80\x03\n\rArchV2Request\x12\x33\n\x0c\x63reatePolicy\x18\x01 \x01(\x0b\x32\x1b.archv2.CreatePolicyRequestH\x00\x12;\n\x0bsecretStore\x18\x02 \x01(\x0b\x32$.secret_store_api.SecretStoreRequestH\x00\x12;\n\x0cupdateStatus\x18\x03 \x01(\x0b\x32#.archv2.UpdateDataRoomStatusRequestH\x00\x12?\n\x12getVerificationKey\x18\x04 \x01(\x0b\x32!.archv2.GetVerificationKeyRequestH\x00\x12\x37\n\x0e\x63reateDataRoom\x18\n \x01(\x0b\x32\x1d.archv2.CreateDataRoomRequestH\x00\x12;\n\x10retrieveDataRoom\x18\x0b \x01(\x0b\x32\x1f.archv2.RetrieveDataRoomRequestH\x00\x42\t\n\x07request\"\x9b\x03\n\x0e\x41rchV2Response\x12\x11\n\x07\x66\x61ilure\x18\x01 \x01(\tH\x00\x12\x34\n\x0c\x63reatePolicy\x18\x02 \x01(\x0b\x32\x1c.archv2.CreatePolicyResponseH\x00\x12<\n\x0bsecretStore\x18\x03 \x01(\x0b\x32%.secret_store_api.SecretStoreResponseH\x00\x12<\n\x0cupdateStatus\x18\x04 \x01(\x0b\x32$.archv2.UpdateDataRoomStatusResponseH\x00\x12@\n\x12getVerificationKey\x18\x05 \x01(\x0b\x32\".archv2.GetVerificationKeyResponseH\x00\x12\x38\n\x0e\x63reateDataRoom\x18\n \x01(\x0b\x32\x1e.archv2.CreateDataRoomResponseH\x00\x12<\n\x10retrieveDataRoom\x18\x0b \x01(\x0b\x32 .archv2.RetrieveDataRoomResponseH\x00\x42\n\n\x08response\"\x1b\n\x19GetVerificationKeyRequest\"5\n\x1aGetVerificationKeyResponse\x12\x17\n\x0fverificationKey\x18\x01 \x01(\x0c\"]\n\x15\x43reateDataRoomRequest\x12\x10\n\x08\x64\x61taRoom\x18\x01 \x01(\t\x12\x1d\n\x10\x64\x61taRoomMetadata\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x13\n\x11_dataRoomMetadata\"~\n\x16\x43reateDataRoomResponse\x12\x14\n\ndataRoomId\x18\x01 \x01(\x0cH\x00\x12\x42\n\x17\x64\x61taRoomValidationError\x18\x02 \x01(\x0b\x32\x1f.archv2.DataRoomValidationErrorH\x00\x42\n\n\x08response\"f\n\x13\x43reatePolicyRequest\x12\x31\n\x06policy\x18\x01 \x01(\x0b\x32!.release_policy_api.ReleasePolicy\x12\x12\n\x05scope\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x42\x08\n\x06_scope\"(\n\x14\x43reatePolicyResponse\x12\x10\n\x08policyId\x18\x01 \x01(\t\"Y\n\x1bUpdateDataRoomStatusRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\t\x12&\n\x06status\x18\x02 \x01(\x0e\x32\x16.archv2.DataRoomStatus\"\x1e\n\x1cUpdateDataRoomStatusResponse\"-\n\x17RetrieveDataRoomRequest\x12\x12\n\ndataRoomId\x18\x01 \x01(\x0c\",\n\x18RetrieveDataRoomResponse\x12\x10\n\x08\x64\x61taRoom\x18\x01 \x01(\t\"\xc6\x02\n\x17\x44\x61taRoomValidationError\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x1c\n\x0fpermissionIndex\x18\x02 \x01(\x04H\x00\x88\x01\x01\x12\x1a\n\rcomputeNodeId\x18\x03 \x01(\tH\x01\x88\x01\x01\x12\x1d\n\x10userPermissionId\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\'\n\x1a\x61ttestationSpecificationId\x18\x05 \x01(\tH\x03\x88\x01\x01\x12#\n\x16\x61uthenticationMethodId\x18\x06 \x01(\tH\x04\x88\x01\x01\x42\x12\n\x10_permissionIndexB\x10\n\x0e_computeNodeIdB\x13\n\x11_userPermissionIdB\x1d\n\x1b_attestationSpecificationIdB\x19\n\x17_authenticationMethodId*)\n\x0e\x44\x61taRoomStatus\x12\n\n\x06\x41\x63tive\x10\x00\x12\x0b\n\x07Stopped\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'archv2_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_DATAROOMSTATUS']._serialized_start=1882
  _globals['_DATAROOMSTATUS']._serialized_end=1923
  _globals['_ARCHV2REQUEST']._serialized_start=84
  _globals['_ARCHV2REQUEST']._serialized_end=468
  _globals['_ARCHV2RESPONSE']._serialized_start=471
  _globals['_ARCHV2RESPONSE']._serialized_end=882
  _globals['_GETVERIFICATIONKEYREQUEST']._serialized_start=884
  _globals['_GETVERIFICATIONKEYREQUEST']._serialized_end=911
  _globals['_GETVERIFICATIONKEYRESPONSE']._serialized_start=913
  _globals['_GETVERIFICATIONKEYRESPONSE']._serialized_end=966
  _globals['_CREATEDATAROOMREQUEST']._serialized_start=968
  _globals['_CREATEDATAROOMREQUEST']._serialized_end=1061
  _globals['_CREATEDATAROOMRESPONSE']._serialized_start=1063
  _globals['_CREATEDATAROOMRESPONSE']._serialized_end=1189
  _globals['_CREATEPOLICYREQUEST']._serialized_start=1191
  _globals['_CREATEPOLICYREQUEST']._serialized_end=1293
  _globals['_CREATEPOLICYRESPONSE']._serialized_start=1295
  _globals['_CREATEPOLICYRESPONSE']._serialized_end=1335
  _globals['_UPDATEDATAROOMSTATUSREQUEST']._serialized_start=1337
  _globals['_UPDATEDATAROOMSTATUSREQUEST']._serialized_end=1426
  _globals['_UPDATEDATAROOMSTATUSRESPONSE']._serialized_start=1428
  _globals['_UPDATEDATAROOMSTATUSRESPONSE']._serialized_end=1458
  _globals['_RETRIEVEDATAROOMREQUEST']._serialized_start=1460
  _globals['_RETRIEVEDATAROOMREQUEST']._serialized_end=1505
  _globals['_RETRIEVEDATAROOMRESPONSE']._serialized_start=1507
  _globals['_RETRIEVEDATAROOMRESPONSE']._serialized_end=1551
  _globals['_DATAROOMVALIDATIONERROR']._serialized_start=1554
  _globals['_DATAROOMVALIDATIONERROR']._serialized_end=1880
# @@protoc_insertion_point(module_scope)
