# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: secret_store.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'secret_store.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12secret_store.proto\x12\x10secret_store_api\"2\n\x10SecretStoreEntry\x12\x0f\n\x07\x63ontent\x18\x01 \x01(\x0c\x12\r\n\x05state\x18\x02 \x01(\x0c\"I\n\x13\x43reateSecretRequest\x12\x32\n\x06secret\x18\x01 \x01(\x0b\x32\".secret_store_api.SecretStoreEntry\"\"\n\x14\x43reateSecretResponse\x12\n\n\x02id\x18\x01 \x01(\t\"/\n\x10GetSecretRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\r\"Y\n\x11GetSecretResponse\x12\x32\n\x06secret\x18\x01 \x01(\x0b\x32\".secret_store_api.SecretStoreEntry\x12\x10\n\x08\x63\x61sIndex\x18\x02 \x01(\x04\";\n\x13RemoveSecretRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x18\n\x10\x65xpectedCasIndex\x18\x02 \x01(\x04\"\'\n\x14RemoveSecretResponse\x12\x0f\n\x07removed\x18\x01 \x01(\x08\"_\n\x16UpdateSecretAclRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0e\n\x06newAcl\x18\x02 \x01(\x0c\x12\x0f\n\x07version\x18\x03 \x01(\r\x12\x18\n\x10\x65xpectedCasIndex\x18\x04 \x01(\x04\"*\n\x17UpdateSecretAclResponse\x12\x0f\n\x07updated\x18\x01 \x01(\x08\"\x9b\x02\n\x12SecretStoreRequest\x12=\n\x0c\x63reateSecret\x18\x01 \x01(\x0b\x32%.secret_store_api.CreateSecretRequestH\x00\x12\x37\n\tgetSecret\x18\x02 \x01(\x0b\x32\".secret_store_api.GetSecretRequestH\x00\x12=\n\x0cremoveSecret\x18\x03 \x01(\x0b\x32%.secret_store_api.RemoveSecretRequestH\x00\x12\x43\n\x0fupdateSecretAcl\x18\x04 \x01(\x0b\x32(.secret_store_api.UpdateSecretAclRequestH\x00\x42\t\n\x07request\"\xa1\x02\n\x13SecretStoreResponse\x12>\n\x0c\x63reateSecret\x18\x01 \x01(\x0b\x32&.secret_store_api.CreateSecretResponseH\x00\x12\x38\n\tgetSecret\x18\x02 \x01(\x0b\x32#.secret_store_api.GetSecretResponseH\x00\x12>\n\x0cremoveSecret\x18\x03 \x01(\x0b\x32&.secret_store_api.RemoveSecretResponseH\x00\x12\x44\n\x0fupdateSecretAcl\x18\x04 \x01(\x0b\x32).secret_store_api.UpdateSecretAclResponseH\x00\x42\n\n\x08responseb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'secret_store_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SECRETSTOREENTRY']._serialized_start=40
  _globals['_SECRETSTOREENTRY']._serialized_end=90
  _globals['_CREATESECRETREQUEST']._serialized_start=92
  _globals['_CREATESECRETREQUEST']._serialized_end=165
  _globals['_CREATESECRETRESPONSE']._serialized_start=167
  _globals['_CREATESECRETRESPONSE']._serialized_end=201
  _globals['_GETSECRETREQUEST']._serialized_start=203
  _globals['_GETSECRETREQUEST']._serialized_end=250
  _globals['_GETSECRETRESPONSE']._serialized_start=252
  _globals['_GETSECRETRESPONSE']._serialized_end=341
  _globals['_REMOVESECRETREQUEST']._serialized_start=343
  _globals['_REMOVESECRETREQUEST']._serialized_end=402
  _globals['_REMOVESECRETRESPONSE']._serialized_start=404
  _globals['_REMOVESECRETRESPONSE']._serialized_end=443
  _globals['_UPDATESECRETACLREQUEST']._serialized_start=445
  _globals['_UPDATESECRETACLREQUEST']._serialized_end=540
  _globals['_UPDATESECRETACLRESPONSE']._serialized_start=542
  _globals['_UPDATESECRETACLRESPONSE']._serialized_end=584
  _globals['_SECRETSTOREREQUEST']._serialized_start=587
  _globals['_SECRETSTOREREQUEST']._serialized_end=870
  _globals['_SECRETSTORERESPONSE']._serialized_start=873
  _globals['_SECRETSTORERESPONSE']._serialized_end=1162
# @@protoc_insertion_point(module_scope)
