# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: microsoft_dsp.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'microsoft_dsp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13microsoft_dsp.proto\x12\rmicrosoft_dsp\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"G\n\x07ZipFile\x12/\n\nsingleFile\x18\x01 \x01(\x0b\x32\x19.microsoft_dsp.SingleFileH\x00\x42\x0b\n\tselection\"\x83\x01\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12%\n\x03raw\x18\x03 \x01(\x0b\x32\x16.microsoft_dsp.RawFileH\x00\x12%\n\x03zip\x18\x04 \x01(\x0b\x32\x16.microsoft_dsp.ZipFileH\x00\x42\x06\n\x04\x66ile\"&\n\nMemberInfo\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\"/\n\x0bSegmentInfo\x12\x12\n\nshort_name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\"\xac\x01\n\x1fMicrosoftDspWorkerConfiguration\x12\'\n\x05input\x18\x01 \x01(\x0b\x32\x18.microsoft_dsp.SinkInput\x12.\n\x0bmember_info\x18\x02 \x01(\x0b\x32\x19.microsoft_dsp.MemberInfo\x12\x30\n\x0csegment_info\x18\x03 \x01(\x0b\x32\x1a.microsoft_dsp.SegmentInfob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'microsoft_dsp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SINGLEFILE']._serialized_start=38
  _globals['_SINGLEFILE']._serialized_end=64
  _globals['_RAWFILE']._serialized_start=66
  _globals['_RAWFILE']._serialized_end=75
  _globals['_ZIPFILE']._serialized_start=77
  _globals['_ZIPFILE']._serialized_end=148
  _globals['_SINKINPUT']._serialized_start=151
  _globals['_SINKINPUT']._serialized_end=282
  _globals['_MEMBERINFO']._serialized_start=284
  _globals['_MEMBERINFO']._serialized_end=322
  _globals['_SEGMENTINFO']._serialized_start=324
  _globals['_SEGMENTINFO']._serialized_end=371
  _globals['_MICROSOFTDSPWORKERCONFIGURATION']._serialized_start=374
  _globals['_MICROSOFTDSPWORKERCONFIGURATION']._serialized_end=546
# @@protoc_insertion_point(module_scope)
