# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: release_policy.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'release_policy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14release_policy.proto\x12\x12release_policy_api\"M\n\rReleasePolicy\x12\x31\n\x02v0\x18\x01 \x01(\x0b\x32#.release_policy_api.ReleasePolicyV0H\x00\x42\t\n\x07version\"\x93\x01\n\x0fReleasePolicyV0\x12\x36\n\x0csecretSource\x18\x01 \x01(\x0b\x32 .release_policy_api.SecretSource\x12H\n\x15\x61llowedComputeContext\x18\x02 \x01(\x0b\x32).release_policy_api.AllowedComputeContext\"U\n\x15\x41llowedComputeContext\x12\x34\n\x03\x64\x63r\x18\x01 \x01(\x0b\x32%.release_policy_api.DcrComputeContextH\x00\x42\x06\n\x04kind\";\n\x11\x44\x63rComputeContext\x12\x12\n\ndataRoomId\x18\x01 \x01(\t\x12\x12\n\ntargetLeaf\x18\x02 \x01(\t\"T\n\x0cSecretSource\x12<\n\x0bsecretStore\x18\x01 \x01(\x0b\x32%.release_policy_api.SecretStoreSourceH\x00\x42\x06\n\x04kind\"%\n\x11SecretStoreSource\x12\x10\n\x08secretId\x18\x01 \x01(\tb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'release_policy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_RELEASEPOLICY']._serialized_start=44
  _globals['_RELEASEPOLICY']._serialized_end=121
  _globals['_RELEASEPOLICYV0']._serialized_start=124
  _globals['_RELEASEPOLICYV0']._serialized_end=271
  _globals['_ALLOWEDCOMPUTECONTEXT']._serialized_start=273
  _globals['_ALLOWEDCOMPUTECONTEXT']._serialized_end=358
  _globals['_DCRCOMPUTECONTEXT']._serialized_start=360
  _globals['_DCRCOMPUTECONTEXT']._serialized_end=419
  _globals['_SECRETSOURCE']._serialized_start=421
  _globals['_SECRETSOURCE']._serialized_end=505
  _globals['_SECRETSTORESOURCE']._serialized_start=507
  _globals['_SECRETSTORESOURCE']._serialized_end=544
# @@protoc_insertion_point(module_scope)
