"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Fatquote(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EPID_FIELD_NUMBER: builtins.int
    DCAP_FIELD_NUMBER: builtins.int
    NITRO_FIELD_NUMBER: builtins.int
    SNP_FIELD_NUMBER: builtins.int
    DCAPMRSIGNER_FIELD_NUMBER: builtins.int
    @property
    def epid(self) -> global___FatquoteEpid: ...
    @property
    def dcap(self) -> global___FatquoteDcap: ...
    @property
    def nitro(self) -> global___FatquoteNitro: ...
    @property
    def snp(self) -> global___FatquoteSnp: ...
    @property
    def dcapMrsigner(self) -> global___FatquoteDcapMrsigner: ...
    def __init__(
        self,
        *,
        epid: global___FatquoteEpid | None = ...,
        dcap: global___FatquoteDcap | None = ...,
        nitro: global___FatquoteNitro | None = ...,
        snp: global___FatquoteSnp | None = ...,
        dcapMrsigner: global___FatquoteDcapMrsigner | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcap", b"dcap", "dcapMrsigner", b"dcapMrsigner", "epid", b"epid", "fatquote", b"fatquote", "nitro", b"nitro", "snp", b"snp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcap", b"dcap", "dcapMrsigner", b"dcapMrsigner", "epid", b"epid", "fatquote", b"fatquote", "nitro", b"nitro", "snp", b"snp"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["fatquote", b"fatquote"]) -> typing.Literal["epid", "dcap", "nitro", "snp", "dcapMrsigner"] | None: ...

global___Fatquote = Fatquote

@typing.final
class FatquoteEpid(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IASRESPONSEBODY_FIELD_NUMBER: builtins.int
    IASCERTIFICATE_FIELD_NUMBER: builtins.int
    IASSIGNATURE_FIELD_NUMBER: builtins.int
    IASROOTCADER_FIELD_NUMBER: builtins.int
    iasResponseBody: builtins.bytes
    iasCertificate: builtins.bytes
    iasSignature: builtins.bytes
    iasRootCaDer: builtins.bytes
    def __init__(
        self,
        *,
        iasResponseBody: builtins.bytes = ...,
        iasCertificate: builtins.bytes = ...,
        iasSignature: builtins.bytes = ...,
        iasRootCaDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["iasCertificate", b"iasCertificate", "iasResponseBody", b"iasResponseBody", "iasRootCaDer", b"iasRootCaDer", "iasSignature", b"iasSignature"]) -> None: ...

global___FatquoteEpid = FatquoteEpid

@typing.final
class FatquoteDcap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCAPQUOTE_FIELD_NUMBER: builtins.int
    TCBINFO_FIELD_NUMBER: builtins.int
    QEIDENTITY_FIELD_NUMBER: builtins.int
    TCBSIGNCERT_FIELD_NUMBER: builtins.int
    QESIGNCERT_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    dcapQuote: builtins.bytes
    tcbInfo: builtins.bytes
    qeIdentity: builtins.bytes
    tcbSignCert: builtins.bytes
    qeSignCert: builtins.bytes
    dcapRootCaDer: builtins.bytes
    def __init__(
        self,
        *,
        dcapQuote: builtins.bytes = ...,
        tcbInfo: builtins.bytes = ...,
        qeIdentity: builtins.bytes = ...,
        tcbSignCert: builtins.bytes = ...,
        qeSignCert: builtins.bytes = ...,
        dcapRootCaDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dcapQuote", b"dcapQuote", "dcapRootCaDer", b"dcapRootCaDer", "qeIdentity", b"qeIdentity", "qeSignCert", b"qeSignCert", "tcbInfo", b"tcbInfo", "tcbSignCert", b"tcbSignCert"]) -> None: ...

global___FatquoteDcap = FatquoteDcap

@typing.final
class FatquoteDcapMrsigner(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCAPQUOTE_FIELD_NUMBER: builtins.int
    TCBINFO_FIELD_NUMBER: builtins.int
    QEIDENTITY_FIELD_NUMBER: builtins.int
    TCBSIGNCERT_FIELD_NUMBER: builtins.int
    QESIGNCERT_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    SIGSTOREEVIDENCE_FIELD_NUMBER: builtins.int
    dcapQuote: builtins.bytes
    tcbInfo: builtins.bytes
    qeIdentity: builtins.bytes
    tcbSignCert: builtins.bytes
    qeSignCert: builtins.bytes
    dcapRootCaDer: builtins.bytes
    @property
    def sigstoreEvidence(self) -> global___SigstoreEvidence: ...
    def __init__(
        self,
        *,
        dcapQuote: builtins.bytes = ...,
        tcbInfo: builtins.bytes = ...,
        qeIdentity: builtins.bytes = ...,
        tcbSignCert: builtins.bytes = ...,
        qeSignCert: builtins.bytes = ...,
        dcapRootCaDer: builtins.bytes = ...,
        sigstoreEvidence: global___SigstoreEvidence | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["sigstoreEvidence", b"sigstoreEvidence"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcapQuote", b"dcapQuote", "dcapRootCaDer", b"dcapRootCaDer", "qeIdentity", b"qeIdentity", "qeSignCert", b"qeSignCert", "sigstoreEvidence", b"sigstoreEvidence", "tcbInfo", b"tcbInfo", "tcbSignCert", b"tcbSignCert"]) -> None: ...

global___FatquoteDcapMrsigner = FatquoteDcapMrsigner

@typing.final
class SigstoreFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    CONTENTS_FIELD_NUMBER: builtins.int
    name: builtins.str
    contents: builtins.bytes
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        contents: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["contents", b"contents", "name", b"name"]) -> None: ...

global___SigstoreFile = SigstoreFile

@typing.final
class SigstoreEvidence(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    V1_FIELD_NUMBER: builtins.int
    @property
    def v1(self) -> global___SigstoreEvidenceV1: ...
    def __init__(
        self,
        *,
        v1: global___SigstoreEvidenceV1 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["sigstore_evidence", b"sigstore_evidence", "v1", b"v1"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["sigstore_evidence", b"sigstore_evidence", "v1", b"v1"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["sigstore_evidence", b"sigstore_evidence"]) -> typing.Literal["v1"] | None: ...

global___SigstoreEvidence = SigstoreEvidence

@typing.final
class SigstoreEvidenceV1(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILES_FIELD_NUMBER: builtins.int
    REKORLOGENTRYJSON_FIELD_NUMBER: builtins.int
    rekorLogEntryJson: builtins.bytes
    """Contains the inclusion proof and the leaf cosign certificate in the transparency log and ultimately the final SHA of the sgxs."""
    @property
    def files(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SigstoreFile]:
        """This list contains the root.json chain, targets.json and trusted_root.json
        (1.root.json, 2.root.json, .., X.root.json, Y.targets.json, trusted_root.json.)
        """

    def __init__(
        self,
        *,
        files: collections.abc.Iterable[global___SigstoreFile] | None = ...,
        rekorLogEntryJson: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["files", b"files", "rekorLogEntryJson", b"rekorLogEntryJson"]) -> None: ...

global___SigstoreEvidenceV1 = SigstoreEvidenceV1

@typing.final
class FatquoteNitro(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COSE_FIELD_NUMBER: builtins.int
    NITROROOTCADER_FIELD_NUMBER: builtins.int
    cose: builtins.bytes
    nitroRootCaDer: builtins.bytes
    def __init__(
        self,
        *,
        cose: builtins.bytes = ...,
        nitroRootCaDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["cose", b"cose", "nitroRootCaDer", b"nitroRootCaDer"]) -> None: ...

global___FatquoteNitro = FatquoteNitro

@typing.final
class FatquoteSnp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REPORTBIN_FIELD_NUMBER: builtins.int
    AMDARKDER_FIELD_NUMBER: builtins.int
    AMDSEVDER_FIELD_NUMBER: builtins.int
    VCEKCRTDER_FIELD_NUMBER: builtins.int
    REPORTDATA_FIELD_NUMBER: builtins.int
    ROUGHTIMEPUBKEY_FIELD_NUMBER: builtins.int
    ROUGHTIMENONCE_FIELD_NUMBER: builtins.int
    SIGNEDTIMESTAMP_FIELD_NUMBER: builtins.int
    DECENTRIQDER_FIELD_NUMBER: builtins.int
    CHIPDER_FIELD_NUMBER: builtins.int
    reportBin: builtins.bytes
    amdArkDer: builtins.bytes
    amdSevDer: builtins.bytes
    vcekCrtDer: builtins.bytes
    reportData: builtins.bytes
    roughtimePubKey: builtins.bytes
    roughtimeNonce: builtins.bytes
    signedTimestamp: builtins.bytes
    decentriqDer: builtins.bytes
    chipDer: builtins.bytes
    def __init__(
        self,
        *,
        reportBin: builtins.bytes = ...,
        amdArkDer: builtins.bytes = ...,
        amdSevDer: builtins.bytes = ...,
        vcekCrtDer: builtins.bytes = ...,
        reportData: builtins.bytes = ...,
        roughtimePubKey: builtins.bytes = ...,
        roughtimeNonce: builtins.bytes = ...,
        signedTimestamp: builtins.bytes = ...,
        decentriqDer: builtins.bytes = ...,
        chipDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["amdArkDer", b"amdArkDer", "amdSevDer", b"amdSevDer", "chipDer", b"chipDer", "decentriqDer", b"decentriqDer", "reportBin", b"reportBin", "reportData", b"reportData", "roughtimeNonce", b"roughtimeNonce", "roughtimePubKey", b"roughtimePubKey", "signedTimestamp", b"signedTimestamp", "vcekCrtDer", b"vcekCrtDer"]) -> None: ...

global___FatquoteSnp = FatquoteSnp

@typing.final
class AttestationSpecification(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INTELEPID_FIELD_NUMBER: builtins.int
    INTELDCAP_FIELD_NUMBER: builtins.int
    AWSNITRO_FIELD_NUMBER: builtins.int
    AMDSNP_FIELD_NUMBER: builtins.int
    INTELDCAPMRSIGNER_FIELD_NUMBER: builtins.int
    @property
    def intelEpid(self) -> global___AttestationSpecificationIntelEpid: ...
    @property
    def intelDcap(self) -> global___AttestationSpecificationIntelDcap: ...
    @property
    def awsNitro(self) -> global___AttestationSpecificationAwsNitro: ...
    @property
    def amdSnp(self) -> global___AttestationSpecificationAmdSnp: ...
    @property
    def intelDcapMrsigner(self) -> global___AttestationSpecificationIntelDcapMrsigner: ...
    def __init__(
        self,
        *,
        intelEpid: global___AttestationSpecificationIntelEpid | None = ...,
        intelDcap: global___AttestationSpecificationIntelDcap | None = ...,
        awsNitro: global___AttestationSpecificationAwsNitro | None = ...,
        amdSnp: global___AttestationSpecificationAmdSnp | None = ...,
        intelDcapMrsigner: global___AttestationSpecificationIntelDcapMrsigner | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["amdSnp", b"amdSnp", "attestation_specification", b"attestation_specification", "awsNitro", b"awsNitro", "intelDcap", b"intelDcap", "intelDcapMrsigner", b"intelDcapMrsigner", "intelEpid", b"intelEpid"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["amdSnp", b"amdSnp", "attestation_specification", b"attestation_specification", "awsNitro", b"awsNitro", "intelDcap", b"intelDcap", "intelDcapMrsigner", b"intelDcapMrsigner", "intelEpid", b"intelEpid"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["attestation_specification", b"attestation_specification"]) -> typing.Literal["intelEpid", "intelDcap", "awsNitro", "amdSnp", "intelDcapMrsigner"] | None: ...

global___AttestationSpecification = AttestationSpecification

@typing.final
class AttestationSpecificationIntelEpid(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MRENCLAVE_FIELD_NUMBER: builtins.int
    IASROOTCADER_FIELD_NUMBER: builtins.int
    ACCEPTDEBUG_FIELD_NUMBER: builtins.int
    ACCEPTGROUPOUTOFDATE_FIELD_NUMBER: builtins.int
    ACCEPTCONFIGURATIONNEEDED_FIELD_NUMBER: builtins.int
    mrenclave: builtins.bytes
    iasRootCaDer: builtins.bytes
    acceptDebug: builtins.bool
    acceptGroupOutOfDate: builtins.bool
    acceptConfigurationNeeded: builtins.bool
    def __init__(
        self,
        *,
        mrenclave: builtins.bytes = ...,
        iasRootCaDer: builtins.bytes = ...,
        acceptDebug: builtins.bool = ...,
        acceptGroupOutOfDate: builtins.bool = ...,
        acceptConfigurationNeeded: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["acceptConfigurationNeeded", b"acceptConfigurationNeeded", "acceptDebug", b"acceptDebug", "acceptGroupOutOfDate", b"acceptGroupOutOfDate", "iasRootCaDer", b"iasRootCaDer", "mrenclave", b"mrenclave"]) -> None: ...

global___AttestationSpecificationIntelEpid = AttestationSpecificationIntelEpid

@typing.final
class AttestationSpecificationIntelDcap(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MRENCLAVE_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    ACCEPTDEBUG_FIELD_NUMBER: builtins.int
    ACCEPTOUTOFDATE_FIELD_NUMBER: builtins.int
    ACCEPTCONFIGURATIONNEEDED_FIELD_NUMBER: builtins.int
    ACCEPTREVOKED_FIELD_NUMBER: builtins.int
    mrenclave: builtins.bytes
    dcapRootCaDer: builtins.bytes
    acceptDebug: builtins.bool
    acceptOutOfDate: builtins.bool
    acceptConfigurationNeeded: builtins.bool
    acceptRevoked: builtins.bool
    def __init__(
        self,
        *,
        mrenclave: builtins.bytes = ...,
        dcapRootCaDer: builtins.bytes = ...,
        acceptDebug: builtins.bool = ...,
        acceptOutOfDate: builtins.bool = ...,
        acceptConfigurationNeeded: builtins.bool = ...,
        acceptRevoked: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["acceptConfigurationNeeded", b"acceptConfigurationNeeded", "acceptDebug", b"acceptDebug", "acceptOutOfDate", b"acceptOutOfDate", "acceptRevoked", b"acceptRevoked", "dcapRootCaDer", b"dcapRootCaDer", "mrenclave", b"mrenclave"]) -> None: ...

global___AttestationSpecificationIntelDcap = AttestationSpecificationIntelDcap

@typing.final
class AttestationSpecificationAwsNitro(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NITROROOTCADER_FIELD_NUMBER: builtins.int
    PCR0_FIELD_NUMBER: builtins.int
    PCR1_FIELD_NUMBER: builtins.int
    PCR2_FIELD_NUMBER: builtins.int
    PCR8_FIELD_NUMBER: builtins.int
    nitroRootCaDer: builtins.bytes
    pcr0: builtins.bytes
    pcr1: builtins.bytes
    pcr2: builtins.bytes
    pcr8: builtins.bytes
    def __init__(
        self,
        *,
        nitroRootCaDer: builtins.bytes = ...,
        pcr0: builtins.bytes = ...,
        pcr1: builtins.bytes = ...,
        pcr2: builtins.bytes = ...,
        pcr8: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["nitroRootCaDer", b"nitroRootCaDer", "pcr0", b"pcr0", "pcr1", b"pcr1", "pcr2", b"pcr2", "pcr8", b"pcr8"]) -> None: ...

global___AttestationSpecificationAwsNitro = AttestationSpecificationAwsNitro

@typing.final
class AttestationSpecificationAmdSnp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AMDARKDER_FIELD_NUMBER: builtins.int
    MEASUREMENT_FIELD_NUMBER: builtins.int
    ROUGHTIMEPUBKEY_FIELD_NUMBER: builtins.int
    AUTHORIZEDCHIPIDS_FIELD_NUMBER: builtins.int
    DECENTRIQDER_FIELD_NUMBER: builtins.int
    amdArkDer: builtins.bytes
    measurement: builtins.bytes
    roughtimePubKey: builtins.bytes
    decentriqDer: builtins.bytes
    @property
    def authorizedChipIds(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(
        self,
        *,
        amdArkDer: builtins.bytes = ...,
        measurement: builtins.bytes = ...,
        roughtimePubKey: builtins.bytes = ...,
        authorizedChipIds: collections.abc.Iterable[builtins.bytes] | None = ...,
        decentriqDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["amdArkDer", b"amdArkDer", "authorizedChipIds", b"authorizedChipIds", "decentriqDer", b"decentriqDer", "measurement", b"measurement", "roughtimePubKey", b"roughtimePubKey"]) -> None: ...

global___AttestationSpecificationAmdSnp = AttestationSpecificationAmdSnp

@typing.final
class AttestationSpecificationIntelDcapMrsigner(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MRSIGNER_FIELD_NUMBER: builtins.int
    DCAPROOTCADER_FIELD_NUMBER: builtins.int
    ACCEPTDEBUG_FIELD_NUMBER: builtins.int
    ACCEPTOUTOFDATE_FIELD_NUMBER: builtins.int
    ACCEPTCONFIGURATIONNEEDED_FIELD_NUMBER: builtins.int
    ACCEPTREVOKED_FIELD_NUMBER: builtins.int
    ISVPRODID_FIELD_NUMBER: builtins.int
    SIGSTOREROOTOLPCJSON_FIELD_NUMBER: builtins.int
    mrsigner: builtins.bytes
    dcapRootCaDer: builtins.bytes
    acceptDebug: builtins.bool
    acceptOutOfDate: builtins.bool
    acceptConfigurationNeeded: builtins.bool
    acceptRevoked: builtins.bool
    isvprodid: builtins.int
    sigstoreRootOlpcJson: builtins.bytes
    def __init__(
        self,
        *,
        mrsigner: builtins.bytes = ...,
        dcapRootCaDer: builtins.bytes = ...,
        acceptDebug: builtins.bool = ...,
        acceptOutOfDate: builtins.bool = ...,
        acceptConfigurationNeeded: builtins.bool = ...,
        acceptRevoked: builtins.bool = ...,
        isvprodid: builtins.int = ...,
        sigstoreRootOlpcJson: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["acceptConfigurationNeeded", b"acceptConfigurationNeeded", "acceptDebug", b"acceptDebug", "acceptOutOfDate", b"acceptOutOfDate", "acceptRevoked", b"acceptRevoked", "dcapRootCaDer", b"dcapRootCaDer", "isvprodid", b"isvprodid", "mrsigner", b"mrsigner", "sigstoreRootOlpcJson", b"sigstoreRootOlpcJson"]) -> None: ...

global___AttestationSpecificationIntelDcapMrsigner = AttestationSpecificationIntelDcapMrsigner
