# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: adform_dsp.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'adform_dsp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x61\x64\x66orm_dsp.proto\x12\nadform_dsp\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"D\n\x07ZipFile\x12,\n\nsingleFile\x18\x01 \x01(\x0b\x32\x16.adform_dsp.SingleFileH\x00\x42\x0b\n\tselection\"}\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\"\n\x03raw\x18\x03 \x01(\x0b\x32\x13.adform_dsp.RawFileH\x00\x12\"\n\x03zip\x18\x04 \x01(\x0b\x32\x13.adform_dsp.ZipFileH\x00\x42\x06\n\x04\x66ile\"q\n\x1c\x41\x64\x66ormDspWorkerConfiguration\x12$\n\x05input\x18\x01 \x01(\x0b\x32\x15.adform_dsp.SinkInput\x12\x16\n\x0esegment_owners\x18\x02 \x03(\t\x12\x13\n\x0bgzip_output\x18\x03 \x01(\x08\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'adform_dsp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SINGLEFILE']._serialized_start=32
  _globals['_SINGLEFILE']._serialized_end=58
  _globals['_RAWFILE']._serialized_start=60
  _globals['_RAWFILE']._serialized_end=69
  _globals['_ZIPFILE']._serialized_start=71
  _globals['_ZIPFILE']._serialized_end=139
  _globals['_SINKINPUT']._serialized_start=141
  _globals['_SINKINPUT']._serialized_end=266
  _globals['_ADFORMDSPWORKERCONFIGURATION']._serialized_start=268
  _globals['_ADFORMDSPWORKERCONFIGURATION']._serialized_end=381
# @@protoc_insertion_point(module_scope)
