"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SnowflakeSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WAREHOUSENAME_FIELD_NUMBER: builtins.int
    DATABASENAME_FIELD_NUMBER: builtins.int
    SCHEMANAME_FIELD_NUMBER: builtins.int
    TABLENAME_FIELD_NUMBER: builtins.int
    STAGENAME_FIELD_NUMBER: builtins.int
    warehouseName: builtins.str
    databaseName: builtins.str
    schemaName: builtins.str
    tableName: builtins.str
    stageName: builtins.str
    def __init__(
        self,
        *,
        warehouseName: builtins.str = ...,
        databaseName: builtins.str = ...,
        schemaName: builtins.str = ...,
        tableName: builtins.str = ...,
        stageName: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["databaseName", b"databaseName", "schemaName", b"schemaName", "stageName", b"stageName", "tableName", b"tableName", "warehouseName", b"warehouseName"]) -> None: ...

global___SnowflakeSource = SnowflakeSource

@typing.final
class DataSourceSnowflakeWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SOURCE_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    credentialsDependency: builtins.str
    @property
    def source(self) -> global___SnowflakeSource: ...
    def __init__(
        self,
        *,
        source: global___SnowflakeSource | None = ...,
        credentialsDependency: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["source", b"source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["credentialsDependency", b"credentialsDependency", "source", b"source"]) -> None: ...

global___DataSourceSnowflakeWorkerConfiguration = DataSourceSnowflakeWorkerConfiguration
