"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class EndorsementRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PKIENDORSEMENTREQUEST_FIELD_NUMBER: builtins.int
    DCRSECRETENDORSEMENTREQUEST_FIELD_NUMBER: builtins.int
    @property
    def pkiEndorsementRequest(self) -> global___PkiEndorsementRequest: ...
    @property
    def dcrSecretEndorsementRequest(self) -> global___DcrSecretEndorsementRequest: ...
    def __init__(
        self,
        *,
        pkiEndorsementRequest: global___PkiEndorsementRequest | None = ...,
        dcrSecretEndorsementRequest: global___DcrSecretEndorsementRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcrSecretEndorsementRequest", b"dcrSecretEndorsementRequest", "endorsementRequest", b"endorsementRequest", "pkiEndorsementRequest", b"pkiEndorsementRequest"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcrSecretEndorsementRequest", b"dcrSecretEndorsementRequest", "endorsementRequest", b"endorsementRequest", "pkiEndorsementRequest", b"pkiEndorsementRequest"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["endorsementRequest", b"endorsementRequest"]) -> typing.Literal["pkiEndorsementRequest", "dcrSecretEndorsementRequest"] | None: ...

global___EndorsementRequest = EndorsementRequest

@typing.final
class EndorsementResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PKIENDORSEMENTRESPONSE_FIELD_NUMBER: builtins.int
    DCRSECRETENDORSEMENTRESPONSE_FIELD_NUMBER: builtins.int
    @property
    def pkiEndorsementResponse(self) -> global___PkiEndorsementResponse: ...
    @property
    def dcrSecretEndorsementResponse(self) -> global___DcrSecretEndorsementResponse: ...
    def __init__(
        self,
        *,
        pkiEndorsementResponse: global___PkiEndorsementResponse | None = ...,
        dcrSecretEndorsementResponse: global___DcrSecretEndorsementResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcrSecretEndorsementResponse", b"dcrSecretEndorsementResponse", "endorsementResponse", b"endorsementResponse", "pkiEndorsementResponse", b"pkiEndorsementResponse"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcrSecretEndorsementResponse", b"dcrSecretEndorsementResponse", "endorsementResponse", b"endorsementResponse", "pkiEndorsementResponse", b"pkiEndorsementResponse"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["endorsementResponse", b"endorsementResponse"]) -> typing.Literal["pkiEndorsementResponse", "dcrSecretEndorsementResponse"] | None: ...

global___EndorsementResponse = EndorsementResponse

@typing.final
class PkiEndorsementRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CERTIFICATECHAINPEM_FIELD_NUMBER: builtins.int
    certificateChainPem: builtins.bytes
    def __init__(
        self,
        *,
        certificateChainPem: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["certificateChainPem", b"certificateChainPem"]) -> None: ...

global___PkiEndorsementRequest = PkiEndorsementRequest

@typing.final
class PkiEndorsementResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PKIENDORSEMENT_FIELD_NUMBER: builtins.int
    @property
    def pkiEndorsement(self) -> global___EnclaveEndorsement: ...
    def __init__(
        self,
        *,
        pkiEndorsement: global___EnclaveEndorsement | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["pkiEndorsement", b"pkiEndorsement"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["pkiEndorsement", b"pkiEndorsement"]) -> None: ...

global___PkiEndorsementResponse = PkiEndorsementResponse

@typing.final
class DcrSecretEndorsementRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCRSECRET_FIELD_NUMBER: builtins.int
    dcrSecret: builtins.str
    def __init__(
        self,
        *,
        dcrSecret: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dcrSecret", b"dcrSecret"]) -> None: ...

global___DcrSecretEndorsementRequest = DcrSecretEndorsementRequest

@typing.final
class DcrSecretEndorsementResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCRSECRETENDORSEMENT_FIELD_NUMBER: builtins.int
    DCRSECRETID_FIELD_NUMBER: builtins.int
    dcrSecretId: builtins.bytes
    @property
    def dcrSecretEndorsement(self) -> global___EnclaveEndorsement: ...
    def __init__(
        self,
        *,
        dcrSecretEndorsement: global___EnclaveEndorsement | None = ...,
        dcrSecretId: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcrSecretEndorsement", b"dcrSecretEndorsement"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcrSecretEndorsement", b"dcrSecretEndorsement", "dcrSecretId", b"dcrSecretId"]) -> None: ...

global___DcrSecretEndorsementResponse = DcrSecretEndorsementResponse

@typing.final
class EnclaveEndorsements(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PERSONALPKI_FIELD_NUMBER: builtins.int
    DQPKI_FIELD_NUMBER: builtins.int
    DCRSECRET_FIELD_NUMBER: builtins.int
    @property
    def personalPki(self) -> global___EnclaveEndorsement: ...
    @property
    def dqPki(self) -> global___EnclaveEndorsement: ...
    @property
    def dcrSecret(self) -> global___EnclaveEndorsement: ...
    def __init__(
        self,
        *,
        personalPki: global___EnclaveEndorsement | None = ...,
        dqPki: global___EnclaveEndorsement | None = ...,
        dcrSecret: global___EnclaveEndorsement | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["dcrSecret", b"dcrSecret", "dqPki", b"dqPki", "personalPki", b"personalPki"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dcrSecret", b"dcrSecret", "dqPki", b"dqPki", "personalPki", b"personalPki"]) -> None: ...

global___EnclaveEndorsements = EnclaveEndorsements

@typing.final
class EnclaveEndorsement(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENDORSEMENTCERTIFICATEDER_FIELD_NUMBER: builtins.int
    endorsementCertificateDer: builtins.bytes
    def __init__(
        self,
        *,
        endorsementCertificateDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["endorsementCertificateDer", b"endorsementCertificateDer"]) -> None: ...

global___EnclaveEndorsement = EnclaveEndorsement

@typing.final
class PkiClaim(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ROOTCERTIFICATEDER_FIELD_NUMBER: builtins.int
    rootCertificateDer: builtins.bytes
    def __init__(
        self,
        *,
        rootCertificateDer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["rootCertificateDer", b"rootCertificateDer"]) -> None: ...

global___PkiClaim = PkiClaim

@typing.final
class DcrSecretClaim(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DCRSECRETID_FIELD_NUMBER: builtins.int
    dcrSecretId: builtins.bytes
    def __init__(
        self,
        *,
        dcrSecretId: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dcrSecretId", b"dcrSecretId"]) -> None: ...

global___DcrSecretClaim = DcrSecretClaim
