"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SecretStoreEntry(google.protobuf.message.Message):
    """An entry in the secret store"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    STATE_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    state: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
        state: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["content", b"content", "state", b"state"]) -> None: ...

global___SecretStoreEntry = SecretStoreEntry

@typing.final
class CreateSecretRequest(google.protobuf.message.Message):
    """/ Add a new secret to the store"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> global___SecretStoreEntry: ...
    def __init__(
        self,
        *,
        secret: global___SecretStoreEntry | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["secret", b"secret"]) -> None: ...

global___CreateSecretRequest = CreateSecretRequest

@typing.final
class CreateSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id"]) -> None: ...

global___CreateSecretResponse = CreateSecretResponse

@typing.final
class GetSecretRequest(google.protobuf.message.Message):
    """/ Get a single secret from the secret store"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    id: builtins.str
    version: builtins.int
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        version: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["id", b"id", "version", b"version"]) -> None: ...

global___GetSecretRequest = GetSecretRequest

@typing.final
class GetSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    CASINDEX_FIELD_NUMBER: builtins.int
    casIndex: builtins.int
    @property
    def secret(self) -> global___SecretStoreEntry: ...
    def __init__(
        self,
        *,
        secret: global___SecretStoreEntry | None = ...,
        casIndex: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["casIndex", b"casIndex", "secret", b"secret"]) -> None: ...

global___GetSecretResponse = GetSecretResponse

@typing.final
class RemoveSecretRequest(google.protobuf.message.Message):
    """/ Remove a particular secret"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    EXPECTEDCASINDEX_FIELD_NUMBER: builtins.int
    id: builtins.str
    expectedCasIndex: builtins.int
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        expectedCasIndex: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["expectedCasIndex", b"expectedCasIndex", "id", b"id"]) -> None: ...

global___RemoveSecretRequest = RemoveSecretRequest

@typing.final
class RemoveSecretResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REMOVED_FIELD_NUMBER: builtins.int
    removed: builtins.bool
    """/ Removal can fail because the expected cas index doesn't match"""
    def __init__(
        self,
        *,
        removed: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["removed", b"removed"]) -> None: ...

global___RemoveSecretResponse = RemoveSecretResponse

@typing.final
class UpdateSecretAclRequest(google.protobuf.message.Message):
    """/ Remove a particular secret"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    NEWACL_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    EXPECTEDCASINDEX_FIELD_NUMBER: builtins.int
    id: builtins.str
    newAcl: builtins.bytes
    version: builtins.int
    expectedCasIndex: builtins.int
    def __init__(
        self,
        *,
        id: builtins.str = ...,
        newAcl: builtins.bytes = ...,
        version: builtins.int = ...,
        expectedCasIndex: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["expectedCasIndex", b"expectedCasIndex", "id", b"id", "newAcl", b"newAcl", "version", b"version"]) -> None: ...

global___UpdateSecretAclRequest = UpdateSecretAclRequest

@typing.final
class UpdateSecretAclResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UPDATED_FIELD_NUMBER: builtins.int
    updated: builtins.bool
    """/ Update can fail because the expected cas index doesn't match"""
    def __init__(
        self,
        *,
        updated: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["updated", b"updated"]) -> None: ...

global___UpdateSecretAclResponse = UpdateSecretAclResponse

@typing.final
class SecretStoreRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATESECRET_FIELD_NUMBER: builtins.int
    GETSECRET_FIELD_NUMBER: builtins.int
    REMOVESECRET_FIELD_NUMBER: builtins.int
    UPDATESECRETACL_FIELD_NUMBER: builtins.int
    @property
    def createSecret(self) -> global___CreateSecretRequest: ...
    @property
    def getSecret(self) -> global___GetSecretRequest: ...
    @property
    def removeSecret(self) -> global___RemoveSecretRequest: ...
    @property
    def updateSecretAcl(self) -> global___UpdateSecretAclRequest: ...
    def __init__(
        self,
        *,
        createSecret: global___CreateSecretRequest | None = ...,
        getSecret: global___GetSecretRequest | None = ...,
        removeSecret: global___RemoveSecretRequest | None = ...,
        updateSecretAcl: global___UpdateSecretAclRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["createSecret", b"createSecret", "getSecret", b"getSecret", "removeSecret", b"removeSecret", "request", b"request", "updateSecretAcl", b"updateSecretAcl"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["createSecret", b"createSecret", "getSecret", b"getSecret", "removeSecret", b"removeSecret", "request", b"request", "updateSecretAcl", b"updateSecretAcl"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["request", b"request"]) -> typing.Literal["createSecret", "getSecret", "removeSecret", "updateSecretAcl"] | None: ...

global___SecretStoreRequest = SecretStoreRequest

@typing.final
class SecretStoreResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATESECRET_FIELD_NUMBER: builtins.int
    GETSECRET_FIELD_NUMBER: builtins.int
    REMOVESECRET_FIELD_NUMBER: builtins.int
    UPDATESECRETACL_FIELD_NUMBER: builtins.int
    @property
    def createSecret(self) -> global___CreateSecretResponse: ...
    @property
    def getSecret(self) -> global___GetSecretResponse: ...
    @property
    def removeSecret(self) -> global___RemoveSecretResponse: ...
    @property
    def updateSecretAcl(self) -> global___UpdateSecretAclResponse: ...
    def __init__(
        self,
        *,
        createSecret: global___CreateSecretResponse | None = ...,
        getSecret: global___GetSecretResponse | None = ...,
        removeSecret: global___RemoveSecretResponse | None = ...,
        updateSecretAcl: global___UpdateSecretAclResponse | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["createSecret", b"createSecret", "getSecret", b"getSecret", "removeSecret", b"removeSecret", "response", b"response", "updateSecretAcl", b"updateSecretAcl"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["createSecret", b"createSecret", "getSecret", b"getSecret", "removeSecret", b"removeSecret", "response", b"response", "updateSecretAcl", b"updateSecretAcl"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["response", b"response"]) -> typing.Literal["createSecret", "getSecret", "removeSecret", "updateSecretAcl"] | None: ...

global___SecretStoreResponse = SecretStoreResponse
