"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _IdFormat:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _IdFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_IdFormat.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNHASHED_EMAIL: _IdFormat.ValueType  # 0
    HASHED_EMAIL_SHA256: _IdFormat.ValueType  # 1
    PHONE_NR: _IdFormat.ValueType  # 2
    HASHED_PHONE_NR_SHA256: _IdFormat.ValueType  # 3

class IdFormat(_IdFormat, metaclass=_IdFormatEnumTypeWrapper): ...

UNHASHED_EMAIL: IdFormat.ValueType  # 0
HASHED_EMAIL_SHA256: IdFormat.ValueType  # 1
PHONE_NR: IdFormat.ValueType  # 2
HASHED_PHONE_NR_SHA256: IdFormat.ValueType  # 3
global___IdFormat = IdFormat

class _Country:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _CountryEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Country.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _Country.ValueType  # 0
    """default value"""
    UNITED_ARAB_EMIRATES: _Country.ValueType  # 1
    AUSTRALIA: _Country.ValueType  # 2
    BELGIUM: _Country.ValueType  # 3
    BRAZIL: _Country.ValueType  # 4
    CANADA: _Country.ValueType  # 5
    GERMANY: _Country.ValueType  # 6
    EGYPT: _Country.ValueType  # 7
    SPAIN: _Country.ValueType  # 8
    FRANCE: _Country.ValueType  # 9
    UNITED_KINGDOM: _Country.ValueType  # 10
    INDIA: _Country.ValueType  # 11
    ITALY: _Country.ValueType  # 12
    JAPAN: _Country.ValueType  # 13
    MEXICO: _Country.ValueType  # 14
    NETHERLANDS: _Country.ValueType  # 15
    POLAND: _Country.ValueType  # 16
    SAUDI_ARABIA: _Country.ValueType  # 17
    SWEDEN: _Country.ValueType  # 18
    SINGAPORE: _Country.ValueType  # 19
    TURKEY: _Country.ValueType  # 20
    UNITED_STATES_OF_AMERICA: _Country.ValueType  # 21

class Country(_Country, metaclass=_CountryEnumTypeWrapper): ...

UNKNOWN: Country.ValueType  # 0
"""default value"""
UNITED_ARAB_EMIRATES: Country.ValueType  # 1
AUSTRALIA: Country.ValueType  # 2
BELGIUM: Country.ValueType  # 3
BRAZIL: Country.ValueType  # 4
CANADA: Country.ValueType  # 5
GERMANY: Country.ValueType  # 6
EGYPT: Country.ValueType  # 7
SPAIN: Country.ValueType  # 8
FRANCE: Country.ValueType  # 9
UNITED_KINGDOM: Country.ValueType  # 10
INDIA: Country.ValueType  # 11
ITALY: Country.ValueType  # 12
JAPAN: Country.ValueType  # 13
MEXICO: Country.ValueType  # 14
NETHERLANDS: Country.ValueType  # 15
POLAND: Country.ValueType  # 16
SAUDI_ARABIA: Country.ValueType  # 17
SWEDEN: Country.ValueType  # 18
SINGAPORE: Country.ValueType  # 19
TURKEY: Country.ValueType  # 20
UNITED_STATES_OF_AMERICA: Country.ValueType  # 21
global___Country = Country

@typing.final
class SingleFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name"]) -> None: ...

global___SingleFile = SingleFile

@typing.final
class RawFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RawFile = RawFile

@typing.final
class ZipFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SINGLEFILE_FIELD_NUMBER: builtins.int
    @property
    def singleFile(self) -> global___SingleFile: ...
    def __init__(
        self,
        *,
        singleFile: global___SingleFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["selection", b"selection", "singleFile", b"singleFile"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["selection", b"selection"]) -> typing.Literal["singleFile"] | None: ...

global___ZipFile = ZipFile

@typing.final
class DataInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPENDENCY_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    RAW_FIELD_NUMBER: builtins.int
    ZIP_FIELD_NUMBER: builtins.int
    dependency: builtins.str
    name: builtins.str
    @property
    def raw(self) -> global___RawFile: ...
    @property
    def zip(self) -> global___ZipFile: ...
    def __init__(
        self,
        *,
        dependency: builtins.str = ...,
        name: builtins.str = ...,
        raw: global___RawFile | None = ...,
        zip: global___ZipFile | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["file", b"file", "raw", b"raw", "zip", b"zip"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["dependency", b"dependency", "file", b"file", "name", b"name", "raw", b"raw", "zip", b"zip"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["file", b"file"]) -> typing.Literal["raw", "zip"] | None: ...

global___DataInput = DataInput

@typing.final
class UnHashedEmail(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___UnHashedEmail = UnHashedEmail

@typing.final
class HashedEmailSha256(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___HashedEmailSha256 = HashedEmailSha256

@typing.final
class TargetResource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONNECTIONID_FIELD_NUMBER: builtins.int
    ADVERTISERID_FIELD_NUMBER: builtins.int
    connectionId: builtins.str
    advertiserId: builtins.str
    def __init__(
        self,
        *,
        connectionId: builtins.str = ...,
        advertiserId: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["advertiserId", b"advertiserId", "connectionId", b"connectionId", "target", b"target"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["advertiserId", b"advertiserId", "connectionId", b"connectionId", "target", b"target"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["target", b"target"]) -> typing.Literal["connectionId", "advertiserId"] | None: ...

global___TargetResource = TargetResource

@typing.final
class AmazonAmcWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INPUT_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    TARGETRESOURCE_FIELD_NUMBER: builtins.int
    AUDIENCEID_FIELD_NUMBER: builtins.int
    AUDIENCENAME_FIELD_NUMBER: builtins.int
    AUDIENCEDESCRIPTION_FIELD_NUMBER: builtins.int
    COUNTRY_FIELD_NUMBER: builtins.int
    IDFORMAT_FIELD_NUMBER: builtins.int
    credentialsDependency: builtins.str
    audienceId: builtins.str
    audienceName: builtins.str
    audienceDescription: builtins.str
    country: global___Country.ValueType
    idFormat: global___IdFormat.ValueType
    @property
    def input(self) -> global___DataInput: ...
    @property
    def targetResource(self) -> global___TargetResource: ...
    def __init__(
        self,
        *,
        input: global___DataInput | None = ...,
        credentialsDependency: builtins.str = ...,
        targetResource: global___TargetResource | None = ...,
        audienceId: builtins.str = ...,
        audienceName: builtins.str = ...,
        audienceDescription: builtins.str | None = ...,
        country: global___Country.ValueType = ...,
        idFormat: global___IdFormat.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_audienceDescription", b"_audienceDescription", "audienceDescription", b"audienceDescription", "input", b"input", "targetResource", b"targetResource"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_audienceDescription", b"_audienceDescription", "audienceDescription", b"audienceDescription", "audienceId", b"audienceId", "audienceName", b"audienceName", "country", b"country", "credentialsDependency", b"credentialsDependency", "idFormat", b"idFormat", "input", b"input", "targetResource", b"targetResource"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["_audienceDescription", b"_audienceDescription"]) -> typing.Literal["audienceDescription"] | None: ...

global___AmazonAmcWorkerConfiguration = AmazonAmcWorkerConfiguration
