# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: azure_blob_storage.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'azure_blob_storage.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x61zure_blob_storage.proto\x12\x12\x61zure_blob_storage\"\x0c\n\nImportRole\":\n\nExportRole\x12,\n\x05input\x18\x01 \x01(\x0b\x32\x1d.azure_blob_storage.SinkInput\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"L\n\x07ZipFile\x12\x34\n\nsingleFile\x18\x01 \x01(\x0b\x32\x1e.azure_blob_storage.SingleFileH\x00\x42\x0b\n\tselection\"\x8d\x01\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12*\n\x03raw\x18\x03 \x01(\x0b\x32\x1b.azure_blob_storage.RawFileH\x00\x12*\n\x03zip\x18\x04 \x01(\x0b\x32\x1b.azure_blob_storage.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xb8\x01\n#AzureBlobStorageWorkerConfiguration\x12\x1d\n\x15\x63redentialsDependency\x18\x01 \x01(\t\x12\x34\n\nimportRole\x18\x02 \x01(\x0b\x32\x1e.azure_blob_storage.ImportRoleH\x00\x12\x34\n\nexportRole\x18\x03 \x01(\x0b\x32\x1e.azure_blob_storage.ExportRoleH\x00\x42\x06\n\x04roleb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'azure_blob_storage_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_IMPORTROLE']._serialized_start=48
  _globals['_IMPORTROLE']._serialized_end=60
  _globals['_EXPORTROLE']._serialized_start=62
  _globals['_EXPORTROLE']._serialized_end=120
  _globals['_SINGLEFILE']._serialized_start=122
  _globals['_SINGLEFILE']._serialized_end=148
  _globals['_RAWFILE']._serialized_start=150
  _globals['_RAWFILE']._serialized_end=159
  _globals['_ZIPFILE']._serialized_start=161
  _globals['_ZIPFILE']._serialized_end=237
  _globals['_SINKINPUT']._serialized_start=240
  _globals['_SINKINPUT']._serialized_end=381
  _globals['_AZUREBLOBSTORAGEWORKERCONFIGURATION']._serialized_start=384
  _globals['_AZUREBLOBSTORAGEWORKERCONFIGURATION']._serialized_end=568
# @@protoc_insertion_point(module_scope)
