# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: amazon_amc.proto
# Protobuf Python Version: 5.28.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    28,
    3,
    '',
    'amazon_amc.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10\x61mazon_amc.proto\x12\namazon_amc\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"D\n\x07ZipFile\x12,\n\nsingleFile\x18\x01 \x01(\x0b\x32\x16.amazon_amc.SingleFileH\x00\x42\x0b\n\tselection\"}\n\tDataInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\"\n\x03raw\x18\x03 \x01(\x0b\x32\x13.amazon_amc.RawFileH\x00\x12\"\n\x03zip\x18\x04 \x01(\x0b\x32\x13.amazon_amc.ZipFileH\x00\x42\x06\n\x04\x66ile\"\x0f\n\rUnHashedEmail\"\x13\n\x11HashedEmailSha256\"J\n\x0eTargetResource\x12\x16\n\x0c\x63onnectionId\x18\x01 \x01(\tH\x00\x12\x16\n\x0c\x61\x64vertiserId\x18\x02 \x01(\tH\x00\x42\x08\n\x06target\"\xc9\x02\n\x1c\x41mazonAmcWorkerConfiguration\x12$\n\x05input\x18\x01 \x01(\x0b\x32\x15.amazon_amc.DataInput\x12\x1d\n\x15\x63redentialsDependency\x18\x02 \x01(\t\x12\x32\n\x0etargetResource\x18\x03 \x01(\x0b\x32\x1a.amazon_amc.TargetResource\x12\x12\n\naudienceId\x18\x04 \x01(\t\x12\x14\n\x0c\x61udienceName\x18\x05 \x01(\t\x12 \n\x13\x61udienceDescription\x18\x06 \x01(\tH\x00\x88\x01\x01\x12$\n\x07\x63ountry\x18\x07 \x01(\x0e\x32\x13.amazon_amc.Country\x12&\n\x08idFormat\x18\x08 \x01(\x0e\x32\x14.amazon_amc.IdFormatB\x16\n\x14_audienceDescription*a\n\x08IdFormat\x12\x12\n\x0eUNHASHED_EMAIL\x10\x00\x12\x17\n\x13HASHED_EMAIL_SHA256\x10\x01\x12\x0c\n\x08PHONE_NR\x10\x02\x12\x1a\n\x16HASHED_PHONE_NR_SHA256\x10\x03*\xc8\x02\n\x07\x43ountry\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x18\n\x14UNITED_ARAB_EMIRATES\x10\x01\x12\r\n\tAUSTRALIA\x10\x02\x12\x0b\n\x07\x42\x45LGIUM\x10\x03\x12\n\n\x06\x42RAZIL\x10\x04\x12\n\n\x06\x43\x41NADA\x10\x05\x12\x0b\n\x07GERMANY\x10\x06\x12\t\n\x05\x45GYPT\x10\x07\x12\t\n\x05SPAIN\x10\x08\x12\n\n\x06\x46RANCE\x10\t\x12\x12\n\x0eUNITED_KINGDOM\x10\n\x12\t\n\x05INDIA\x10\x0b\x12\t\n\x05ITALY\x10\x0c\x12\t\n\x05JAPAN\x10\r\x12\n\n\x06MEXICO\x10\x0e\x12\x0f\n\x0bNETHERLANDS\x10\x0f\x12\n\n\x06POLAND\x10\x10\x12\x10\n\x0cSAUDI_ARABIA\x10\x11\x12\n\n\x06SWEDEN\x10\x12\x12\r\n\tSINGAPORE\x10\x13\x12\n\n\x06TURKEY\x10\x14\x12\x1c\n\x18UNITED_STATES_OF_AMERICA\x10\x15\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'amazon_amc_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_IDFORMAT']._serialized_start=714
  _globals['_IDFORMAT']._serialized_end=811
  _globals['_COUNTRY']._serialized_start=814
  _globals['_COUNTRY']._serialized_end=1142
  _globals['_SINGLEFILE']._serialized_start=32
  _globals['_SINGLEFILE']._serialized_end=58
  _globals['_RAWFILE']._serialized_start=60
  _globals['_RAWFILE']._serialized_end=69
  _globals['_ZIPFILE']._serialized_start=71
  _globals['_ZIPFILE']._serialized_end=139
  _globals['_DATAINPUT']._serialized_start=141
  _globals['_DATAINPUT']._serialized_end=266
  _globals['_UNHASHEDEMAIL']._serialized_start=268
  _globals['_UNHASHEDEMAIL']._serialized_end=283
  _globals['_HASHEDEMAILSHA256']._serialized_start=285
  _globals['_HASHEDEMAILSHA256']._serialized_end=304
  _globals['_TARGETRESOURCE']._serialized_start=306
  _globals['_TARGETRESOURCE']._serialized_end=380
  _globals['_AMAZONAMCWORKERCONFIGURATION']._serialized_start=383
  _globals['_AMAZONAMCWORKERCONFIGURATION']._serialized_end=712
# @@protoc_insertion_point(module_scope)
